#ifndef __hegettim_h__
#define __hegettim_h__

#ifndef __helq_h__
#include <helq.h>
#endif


#ifdef OSK
#ifndef CLK_TCK 			// is defined in /dd/DEFS/time.h
#include <time.h>
#endif
#endif


#if defined( AVR )
#include <avr/io.h>		// needed to initialize the timer
#endif


#ifdef DOSGNU

#ifndef __dj_include_Inline_pc_h_
#include <inlines/pc.h>
#endif // #ifndef __dj_include_Inline_pc_h_

#ifndef __dj_include_sys_farptr_h_
#include <sys/farptr.h>
#endif // #ifndef __dj_include_sys_farptr_h_

#endif



#define EPOCH_INVALID 0L

class CEpoch
	{
public:
	ulong ulE ; // The epoch value : Always UTC since 1.1.1970
public:

	operator long()					// convert the value to long
		{
		return (long)ulE ;
		}

	long Diff( CEpoch epochStart ) const	// Returns seconds since event
		{
		return ulE - epochStart.ulE ;
		} ;

	void Add( ulong ulSec )		// Adds seconds to Events time
		{
		ulE += ulSec ;
		} ;

	void Sub( ulong ulSec )		// Subtracts seconds from Events time
		{
		ulE -= ulSec ;
		} ;

	ulong Time( void )				// only the time of the Day
		{
		return ulE % 86400L ;
		}

	bool Isintime( CEpoch from, CEpoch to );	// See file epoisint.cpp

	bool IsAfter( CEpoch from );	// See file epoisaft.cpp

	bool IsBefore( CEpoch from );	// See file epoisbef.cpp

	void GetActual( void );			// See file epochget.cpp

	CEpoch( void ) {; }				// Standard Constructor

	CEpoch( ulong val )
#ifdef __TURBOC__
;
#else
	{ ulE = val; }				// Number initalize
#endif

	CEpoch(uint yr,uint mn,uint dy,uint hr,uint mi,uint sc );
									// See file epochepo.cpp

	void Todate( uint *pyr, uint *pmth, uint *pday, uint *phur, uint *pmin, uint *psec) const ;
									// See file etodate.cpp

	void Txt(				// convert epoch value to date text as "2002/12/31 14:56:55", size=20Bytes
		char *dpDest,
		int maxlen ) const ;
									// See file epochtxt.cpp

	} ;



// get set a file date, where the file name is given
CEpoch getfndat(const char *files_name );
void setfndat(const char *files_name, CEpoch epoch);

// get set a file date of an open file
CEpoch getfdate(tfh fd);
void setfdate(tfh fd, CEpoch epoch);

int		asctotim(					// ASCII ANSI code to daycount() number
			const char **ppch,		// input string
			CEpoch *where );		 // where to save result


struct sdate {
	uint year ;
	uint month ;
	uint day ;
	uint weekday ;	   // 0 = Sunday ...
	} ;
struct sday {
	uint hour ;
	uint minute ;
	uint second ;
	uint msecond ;	   // 1/1000 of seconds
	} ;


struct sdate_uc {
	ulong day ;
	uchar hour ;
	uchar minute ;
	uchar second ;
	} ;

extern void tsdaytim(void);
extern void tsdaytim_open(void);


#if defined( __TURBOC__ ) || defined( DOSGNU ) || defined(MDOS10) || defined(MSVC10)
struct stickuchars {
	uchar ucl ;
	uchar uch ;
	uword uwh ; } ;
struct stickuwords {
	uword uwl ;
	uword uwh ; } ;
struct stickulongs {
	ulong ull ;
	} ;

union uticks {
	struct stickulongs ul ;
	struct stickuwords uw ;
	struct stickuchars uc ;
		} ;


#if defined( __TURBOC__ )  || defined(MDOS10) || defined(MSVC10)
uword far * const tick_in_dos_page_46 = (uword far *)0x0000046C ;
uword far * const hitick_in_dos_page_46 = (uword far *)0x0000046E ;

#define ul_mod_tick_in_dos_page_46	(*(ulong far *)0x0000046C)
#endif
// ... continued #if defined( __TURBOC__ ) || defined( DOSGNU )

static union uticks pre ;
static union uticks post ;
static uword olddistick ;

FUNCT inline void tickopen( void )		// if defined( __TURBOC__ ) || defined( DOSGNU )  || defined(MDOS10) || defined(MSVC10)
{
disable();
outportb( 0x43, 0x34 );
// 0x34 = 0b00110100
//			00		 Select Counter 0
//			  11	 Reload timer lsb msb
//				010	 Mode 2 Rate Generator
//				   0 Binary, not BCD
outportb( 0x40, 255 );	// Low Timer Value
outportb( 0x40, 255 );	// High Timer Value
enable();
}


// Warning : After Midnight the cell 46c will be reset to 0 !
// this is not handled in this program
FUNCT inline ulong gettick( void )		// if defined( __TURBOC__ ) || defined( DOSGNU )
{

disable();			// Interrupt off

#if defined( __TURBOC__ ) || defined(MDOS10) || defined(MSVC10)
pre.uw.uwh = *tick_in_dos_page_46 ;
#else
pre.uw.uwh = _farnspeekw( 0x0000046C ) ;
#endif
outportb( 0x43, 0 );	// Latch counter
pre.uc.ucl = inportb( 0x40 );
pre.uc.uch = inportb( 0x40 );
enable();			// interrupt on

#if defined( __TURBOC__ )  || defined(MDOS10) || defined(MSVC10)
post.uw.uwh = *tick_in_dos_page_46 ;
#else	// DOSGNU = DJGPP
post.uw.uwh = _farnspeekw( 0x0000046C ) ;
#endif

if ( pre.uw.uwh != post.uw.uwh )
	{
	post.uw.uwl = 0 ;
	return post.ul.ull ;
	}
pre.uw.uwl = ~pre.uw.uwl ;
return pre.ul.ull ;
}

						// if defined( __TURBOC__ ) || defined( DOSGNU )  || defined(MDOS10) || defined(MSVC10)
#define getwordtick( result )				\
outportb( 0x43, 0 );							\
((uchar *)&result)[0] = inportb( 0x40 );	\
((uchar *)&result)[1] = inportb( 0x40 );

// ... continued #if defined( __TURBOC__ ) || defined( DOSGNU )


#if defined( __TURBOC__ )

#define getdistick( result )					\
outp( 0x43, 0 );								\
((uchar *)&result)[0] = ~inportb( 0x40 );		\
((uchar *)&result)[1] = ~inportb( 0x40 );		\
if ( ((uword *)&result)[0] < olddistick )		\
	{											\
	if ( ++*tick_in_dos_page_46 == 0 )			\
		{										\
		++*hitick_in_dos_page_46 ;				\
		}										\
	}											\
olddistick = ((uword *)&result)[0] ;			\
((uword *)&result)[1] = *tick_in_dos_page_46 ;

#else			// DOSGNU

#define getdistick( result )					\
outp( 0x43, 0 );								\
((uchar *)&result)[0] = ~inportb( 0x40 );		\
((uchar *)&result)[1] = ~inportb( 0x40 );		\
if ( ((uword *)&result)[0] < olddistick )		\
	{											\
	short tck ;									\
	tck = _farnspeekw( 0x0000046C )+1 ;			\
	_farnspokew( 0x0000046C, tck  ) ;			\
	if ( tck == 0 )								\
		{										\
		tck = _farnspeekw( 0x0000046C+2 )+1 ;	\
		_farnspokew( 0x0000046C+2, tck	) ;		\
		}										\
	}											\
olddistick = ((uword *)&result)[0] ;			\
((uword *)&result)[1] = _farnspeekw( 0x0000046C ) ;


#endif

// ... continued #if defined( __TURBOC__ ) || defined( DOSGNU )

/*
FUNCT inline ulong difftick( ulong start, ulong act )
{
return act - start ;
}
*/
/*
FUNCT inline ulong difftick( ulong start )
{
return gettick() - start ;
}
*/

				// The macro is faster than the inline
#define difftick( start, actual ) ( (actual)-(start) )
#define diffdistick( start, actual ) ( (actual)-(start) )
#define addtick( start, plus ) ( (start)+(plus) )

// As windows 95 counts to negative direction:
#define diffwordtick( start, actual ) ( (start)-(actual) )
#define addwordtick( start, plus ) ( (start)-(plus) )
// #define diffwordtick( start, actual ) ( (actual)-(start) )
// #define addwordtick( start, plus ) ( (start)+(plus) )

#define TICKPERMSEC 1192.5	// 4.77 Mhz / 4

#define ticktomsec(x) ((double)(x)/TICKPERMSEC)

#define msectotick(msec) (ulong)( (msec) * TICKPERMSEC + 0.99999900 )


#define tickpersec 1192500.0	// 4.77 Mhz / 4
#define TICKPERSEC 1192500

#define ticktosec(x) ((double)(x)/tickpersec)


#define sectotick(sec) (ulong)( (sec) * tickpersec + 0.99999900 )

#endif		// if defined( __TURBOC__ ) || defined( DOSGNU )




#if defined( MSVC40 ) || defined( MDOS40 )

// There is no possibility to get the DOS tick counter from 463
// So use the getmsec instead tick
// If you need more accurate timers, then use DOS6.22
// TurboC++(Version 1.0)

#define getdistick(x) x = getmsec();
#define diffdistick diffmsec


#define gettick getmsec		// defined( MDOS40 ) : gettick reads milliseconds - use getmsec
#define difftick diffmsec
#define addtick addmsec
#define ticktomsec(x) (double)(x)
#define msectotick(x) (ulong)(x)
#define ticktosec(x) ((double)(x)/1000.0)
#define sectotick(x) (ulong)((x)*1000.0)
const double tickpersec = 1000.0 ;
#define TICKPERSEC 1000
#define TICKPERMSEC 1
FUNCT inline void tickopen( void )
	{;
	}
#endif


#if defined LINUX

#define tickopen()
#define ticktomsec(x) (double)(x)
#define getwordtick(r) r=(uword)getmsec()
#define diffwordtick(s,a) (uword)diffmsec(s,a)
#define gettick getmsec		// defined( LINUX ) : gettick reads milliseconds - use getmsec
#define difftick diffmsec
#define addtick addmsec
#define ticktomsec(x) (double)(x)
#define msectotick(x) (ulong)(x)
#define ticktosec(x) ((double)(x)/1000.0)
#define sectotick(x) (ulong)((x)*1000.0)
const double tickpersec = 1000.0 ;
#define TICKPERSEC 1000
#endif



#ifdef AVR

// In the makedefs file /home/cc/avrbat/makedefs there is 
// GETTICK_TIMER = 0 Use the 8 Bit Timer
// GETTICK_TIMER = 1 Use the 16 Bit Timer


// The timer settings are done in getticks.cpp tickopen();

FUNCT inline void tickopen(void)
{
// Initialize the Timer
#if GETTICK_TIMER == 0
TCCR0 = AVR_TIMER_DIVIDER_CODE ;		//
					// ATMEGA32 : 
					// Bit 7 : No forced compare
					// Bit 3,6 = 0 : normal, no PWM
					// Bit 5,4 : no connection to output line
					// Bit 2,1,0 : 000 = Stopped
					//			   001 = CK/1
					//			   010 = CK/8
					//			   011 = CK/64 <<<
					//			   100 = CK/256
					//			   101 = CK/1024
					//			   110 = extern H-L
					//			   111 = extern L_H
#endif
#if GETTICK_TIMER == 1
TCCR1A = 0 ;		// Bit 7,6 : no connection to output line
					// Bit 1,0 : no PWM operation
TCCR1B = AVR_TIMER_DIVIDER_CODE ;		//
					// Bit 7 : 0 = No Input noise capture
					// Bit 6 : Edge Select
					// Bit 3 : 0 = No Clear on compare
					// Bit 2,1,0 : 000 = Stopped
					//			   001 = CK/1
					//			   010 = CK/8
					//			   011 = CK/64 <<<
					//			   100 = CK/256
					//			   101 = CK/1024
					//			   110 = extern H-L
					//			   111 = extern L_H
#endif
}



#if AVR_TIMER_DIVIDER_CODE == 1
#define AVR_TIMER_DIVIDER 1
#elif AVR_TIMER_DIVIDER_CODE == 2
#define AVR_TIMER_DIVIDER 8
#elif AVR_TIMER_DIVIDER_CODE == 3
#define AVR_TIMER_DIVIDER 64
#elif AVR_TIMER_DIVIDER_CODE == 4
#define AVR_TIMER_DIVIDER 256
#elif AVR_TIMER_DIVIDER_CODE == 5
#define AVR_TIMER_DIVIDER 1024L
#endif

#define tickpersec ((double)(F_CPU)/(double)AVR_TIMER_DIVIDER)
#define TICKPERSEC (F_CPU/AVR_TIMER_DIVIDER)
#define tickpermsec ((double)(F_CPU)/(double)AVR_TIMER_DIVIDER/1000.0)
#define TICKPERMSEC (F_CPU/AVR_TIMER_DIVIDER/1000L)
#define TICKPER10MSEC (F_CPU/AVR_TIMER_DIVIDER/100L)
#define TICKPER100MSEC (F_CPU/AVR_TIMER_DIVIDER/10L)


extern ulong gettick(void);
extern void tickopen(void);
				// The macro is faster than the inline
#define difftick( start, actual ) ( (actual)-(start) )
#define diffdistick( start, actual ) ( (actual)-(start) )
#define diffwordtick( start, actual ) ( (actual)-(start) )
#define addtick( start, plus ) ( (start)+(plus) )

// The follow lines are calculated by the compiler, if the argument is any constant or constant expression
// The rounding is a bit less than 1.0, see 
// file:///R:/home/html/hjh/cc/english/lq/float.htm "Range of Significand"

#define sectotick( secs ) (ulong)( (secs) * tickpersec + 0.99999900 )
#define msectotick( msecs ) (ulong)( (msecs) * tickpermsec + 0.99999900 )


#if GETTICK_TIMER == 0
extern void getwordtick( uword &result );
#endif
#if GETTICK_TIMER == 1
#define getwordtick( result )				\
result = TCNT1 ;		// Read the 16 bit value ( 2MHz ) from counter 1
#endif


// /////////////////////////////////////////////////////////////
#ifdef AVR
FUNCT inline ulong getmsec()
{
extern ulong g_msec ;		// is maintained in tsdaytim.cpp

return g_msec ;
}

#endif


#endif		// end of the AVR special part


// Millisecond timer
ulong getmsec(void) ;				// gets milliseconds ... somehow
ulong diffmsec( ulong start ) ;				// gets difference since last call
ulong diffmsec( ulong start,ulong actmsec) ; // gets difference since last call
ulong addmsec( ulong start, ulong plus);	 // adds even correct after midnight

void dlmsec(ulong milliseconds);
	// converstoonmsef to text see Cst::msecst() file cstmsec.cpp
// Microsecond timer
ulong getusec(void) ;				// gets micro seconds ... somehow
ulong diffusec( ulong start ) ;				// gets difference since last call
ulong diffusec( ulong start,ulong actusec) ; // gets difference since last call
ulong addusec( ulong start, ulong plus);	 // adds even correct after midnight

ulong getsecmn(void) ;				// gets seconds after midnight ... somehow


// raw maschine ticks, for PC is 4.77 MHz / 4 = 1192500 Hz
// Only for DOS machines
uword getticks_disabled( void ) ;	// call this only when interrupt is disabled

void delaytick(ulong n);					// wait n ticks, call tsnext while waiting
// nanoseconds delay
void dlnsec(ulong nanoseconds);
void caldlnsec( bool bReport = false );


// get date and clock time in UTC (GMT) time
void getutcdatetime( struct sdate *pdate, struct sday *pday );

// get date and clock time in local time
void getlocaldatetime( struct sdate *pdate, struct sday *pday );

// get time zome information
int gettzname ( const char **ppTimezonename, long *pulBias );

#define getenatick(x) x=gettick()
#endif // #if defined( _hegettim_h__ )

