#ifndef __heimath_h__
#define __heimath_h__
// Herbert's (hjherbert@t-online.de) integer math header and prototypes

#include <heport.h>		// defintion of ulong ...

#define NaN (0.0/0.0)

ulong gcd(ulong x, ulong y);	// greatest common divisor
ulong scm(ulong x, ulong y);	// smallest common multiple

uword lsqrt(ulong x);			// Square root of a long integer

extern ulong primelist[] ;		// list of prime values, end = 0

// factorize a number
int primfact(				// returns 0 if O.K.
		ulong x,			// this number
		ulong *pprimfact,	// save there
		uint maxprimfact,	// size of array
		uint *pnprimfact		// save actually found count of factors
		);

sword swmuldiv( sword x, sword y, sword d ) ;
uword uwmuldiv( uword x, uword y, uword d ) ;


extern "C" {
ulong mul_uw_uw( uword x , uword y );	// special optimized for ATMEL
slong mul_sw_sw( sword x , sword y );	// special optimized for ATMEL

uword div_ul_uw( ulong x , uword y, uword *pr = 0 );	// special optimized for ATMEL

 } ;


#define uchar_MAX 255
#define schar_MAX 127
#define schar_MIN -128

#define sword_MIN (sword)0x8000
#define sword_MAX (sword)0x7FFF
#define uword_MAX (uword)0xFFFF


// Extract unsigned Bytes from other types 
#define BYTE0(x) ((x)&0xff)
#define BYTE1(x) ((x>>8)&0xff)
#define BYTE2(x) ((x>>16)&0xff)
#define BYTE3(x) ((x>>24)&0xff)


#if INTSIZE == 16
const sint sint_MAX = 0x7FFF ;
const uint uint_MAX = 0xFFFF ;
#else
// const sint sint_MAX = 0x7FFFFFFF ;
#define sint_MAX (sint)(0x7FFFFFFF)
#define uint_MAX (uint)(0xFFFFFFFF)
#endif

#define slong_MIN (slong )0x80000000L
#define slong_MAX (slong )0x7FFFFFFFL
#define ulong_MAX (ulong )0xFFFFFFFFL



extern void CRC16_Init( void );
extern void CRC16_Update( uchar val );
extern uword CRC16_result( void );

#endif
