#ifndef __tsinout_h__
#define __tsinout_h__


#ifndef __tshar_h__
#include <tshar.h>
#endif


#ifdef AVR
#include <avr/pgmspace.h>	// access to program memory
#endif

// tsinout.h : The timesharing tshar maps these calls to per task calls, using index tshar_task_no
extern uchar in_char( void ); // wait until a character is typed in,reutn it.
extern void out_char( const uchar c );		// Output a character

extern bool in_check( void );	// check if a character is typed in
// the device independ composed functions


#ifdef AVR
extern void out_puts_P( PGM_P p ); // Output / send a string out of program memory lq/otputs.cpp
#define out_puts(x) out_puts_P( PSTR(x) )
#else
extern void out_puts( const char *p ); // Output / send a string lq/otputs.cpp
#endif

extern void out_putsv( const char *p ); // Output / send a string from data memory lq/otputs.cpp

extern void out_nl( void );		// Output / send new line


// read an unsigned char, in hex format, no prompt, no end character, 
// just read one byte '0'..'9' 'A'..'F' and a second byte
extern int in_hex2( uchar *p );



// read an unsigned word, in hexadecimale format, e.g. C3
#ifdef AVR
extern char in_hex8_P(
		PGM_P prompt,	// send / display this before reading characters
		uchar *p );		// Save the converted hex number here
#define in_hex8(x,y) in_hex8_P(PSTR(x),y)
#else
extern char in_hex8(
		const char *prompt,		// send / display this before reading characters
		uchar *p );		// Save the converted hex number here
#endif


// read an unsigned word, in hexadecimale format, e.g. 7FA0
#ifdef AVR
extern char in_hex16_P(
		PGM_P prompt,	// send / display this before reading characters
		uword *p );		// Save the converted hex number here
#define in_hex16(x,y) in_hex16_P(PSTR(x),y)
#else
extern char in_hex16(
		const char *prompt,		// send / display this before reading characters
		uword *p );		// Save the converted hex number here
#endif


// read an unsigned long, in hexadecimale format, e.g. 3DF07FA0
#ifdef AVR
extern char in_hex32_P(
		PGM_P prompt,	// send / display this before reading characters
		ulong *p );		// Save the converted hex number here
#define in_hex32(x,y) in_hex32_P(PSTR(x),y)
#else
extern char in_hex32(
		const char *prompt,		// send / display this before reading characters
		ulong *p );		// Save the converted hex number here
#endif


// Decimale input and output
// read an unsigned char, in decimale format e.g. 226
#ifdef AVR
extern char in_uint8_P(
		PGM_P prompt,	// send / display this before reading characters
		uchar *p );		// Save the converted hex number here
#define in_uint8(x,y) in_uint8_P(PSTR(x),y)
#else
extern char in_uint8(
		const char *prompt,		// send / display this before reading characters
		uchar *p );		// Save the converted hex number here
#endif

// read an unsigned word, in decimale format, e.g. 40200
#ifdef AVR
extern char in_uint16_P(
		PGM_P prompt,	// send / display this before reading characters
		uword *p );		// Save the converted hex number here
#define in_uint32(x,y) in_uint32_P(PSTR(x),y)
#else
extern char in_uint16(
		const char *prompt,		// send / display this before reading characters
		uword *p );		// Save the converted hex number here
#endif


// read an unsigned long, in decimale format, e.g. 1000240
#ifdef AVR
extern char in_uint32_P(
		PGM_P prompt,	// send / display this before reading characters
		ulong *p );		// Save the converted hex number here
#define in_uint16(x,y) in_uint16_P(PSTR(x),y)
#else
extern char in_uint32(
		const char *prompt,		// send / display this before reading characters
		ulong *p );		// Save the converted hex number here
#endif




// read an ssigned char, in decimale format e.g. -100
#ifdef AVR
extern char in_int8_P(
		PGM_P prompt,	// send / display this before reading characters
		schar *p );		// Save the converted hex number here
#define in_int8(x,y) in_int8_P(PSTR(x),y)
#else
extern char in_int8(
		const char *prompt,		// send / display this before reading characters
		schar *p );		// Save the converted hex number here
#endif

// read an signed word, in decimale format, e.g. -16000
#ifdef AVR
extern char in_int16_P(
		PGM_P prompt,	// send / display this before reading characters
		sword *p );		// Save the converted hex number here
#define in_int32(x,y) in_int32_P(PSTR(x),y)
#else
extern char in_int16(
		const char *prompt,		// send / display this before reading characters
		sword *p );		// Save the converted hex number here
#endif


// read an signed long, in decimale format, e.g. -1000240
#ifdef AVR
extern char in_int32_P(
		PGM_P prompt,	// send / display this before reading characters
		slong *p );		// Save the converted hex number here
#define in_int16(x,y) in_int16_P(PSTR(x),y)
#else
extern char in_int32(
		const char *prompt,		// send / display this before reading characters
		slong *p );		// Save the converted hex number here
#endif


// read a float (float), in decimale format e.g. -1.23e-3 or 24.33
#ifdef AVR
extern char in_float_P(
		PGM_P prompt,	// send / display this before reading characters
		float *p );		// Save the converted hex number here
#define in_float(x,y) in_float_P(PSTR(x),y)
#else
extern char in_float(
		const char *prompt,		// send / display this before reading characters
		float *p );		// Save the converted hex number here
#endif






// Display values hex
extern void out_hex1( uchar h );	// Output a single hex digit
extern void out_hex2( uchar h );	// Output two      hex digits
extern void out_hex4( uword h );	// Output four     hex digits
extern void out_hex8( ulong h );	// Output eight    hex digits

// Display several number data types decimale

extern void out_float( float h );	// Output float number
#ifdef AVR							//  sizeof( float ) == sizeof( double )
#define out_double(h) out_float(h)
#else
extern void out_double( double h );	// Output double number
#endif

extern void out_bool( bool h );			// Output F or T and never anything else
extern void out_uint8( uchar h );		// Output 1 to 3 decimale digits (0...255)
extern void out_uint16( uword h );	// Output 1 to 5 decimale digits (0...65535)
extern void out_uint32( ulong h );	// Output 1 to 10 decimale digits (0...4294967295)

extern void out_int8( schar h );		// Output 1 to 3 decimale digits (-128..127)
extern void out_int16( sword h );		// Output 1 to 5 decimale digits (-32768...32767)
extern void out_int32( slong h );		// Output 1 to 10 decimale digits (-2147483647...2147483648)

extern void out_uwfx( uword h, uword prec );			// Output unsigned word fixed point 0...0.999999
extern void out_uwfx( uword h, uword prec, uword n );	// Output unsigned word fixed point 0...n.999999

#define NIN_CHARBUFFER 20
extern char in_charbuffer[NPRGPATH][NIN_CHARBUFFER] ;
// read a line of text into in_charbuffer
extern char in_line( void );		// Save the string in in_charbuffer


#endif // __tsinout_h__
