
; .include "c:/home/cc/include/longbr.inc"
.include "../../../include/longbr.inc"

; Author : HJHerbert (2007-02-07)
; modified HJH20070305 to 20070415 : all results are similar as the Intel Pentium Hardware, expect
; for NaN: If this versio nof addsf3 delivers NaN, then it delivers 0x7FC00000 whenever a NaN 
; condition is met:
; a has Exponent == 0xFF and M22..M0 != 0
; b has Exponent == 0xFF and M22..M0 != 0
; a = +Inf (7F800000) and b = -Inf (FF800000)
; a = -Inf (FF800000) and b = +Inf (7F800000)
;
; Usage: Example
; float fa = -17.444;
; float fb = 306.2;
; ...
; hr1 = fa + fb ;
;
; needs 1000 cycles = 10.5 usec at 16 MHz
;
; The original library routine needed 949 cycles, 59.3125 usec
;
; Arithmetic Errors: less than 0.2 ppm of ( max(abs(a),abs(b),abs(r) )
;
; Example of usage from CPP :
; float r, a, b ;
; r = a + b
; The mantissa a is in the range 0.5 ... 0.9999xxx
; The mantissa b is in the range 0.5 ... 0.9999xxx
; So after a possible neccesary shift the mantisa of the smaller value 
; (the value with the smaller exponent is between 0.0000 and 0.999999
; The result of a mantissa add is between 0.5000000 and 1.99999xxxx
; So after addition normation might be neccesary only once by 1 shift
; There is no need of a loop, but needs to check if after this shift
; the exponent overflows.
;

	round = 1 			; disable (0) or enable (1) rounding
	nan_support = 1		; disable (0) or enable (1) nan input support
	denorm_support = 1	;
	negative_0_support = 1	
; Only if all these 4 labels are set = 1 then the result is the same as by the 
; floating point coprocessor Intel for all valid numbers and subnormals
; Just the Nan is treated different: There is only one NaN its code is 0x7FC00000
; There is no negative NaN
;
;
; denorm_support
; without denorm_support, values below 0x00800000 are considered as 0
; without denorm_support the minimul value nest to 0 is 

	.data

	.text

; r = a + b
; (See also /home/html/hjh/atmel/lib/float.htm)
; Input : 
; r1 = 0 ;	The C Compilers convention
; r25 r24 r23 r22 = a, extended by r13 r12
; r21 r20 r19 r18 = b, extended by r15 r14
; Output :
; r25 24 r23 r22 = r The result, (IEEE)
; register usage :
; r8 : separated exponent of b
; r17 : separated exponent of a
; r16 : The exponent difference
; r1 : 0
;
	.global __subsf3	; Floating point subtraction
__subsf3:
 	subi	r21,0x80	; toggle sign bit of b
	.global __addsf3	; Floating point addition
__addsf3:
 	push	r21
 	push	r20
 	push	r19
 	push	r18
 	push	r17
 	push	r16
	push	r15
	push	r13
	push	r14
	push	r12
	push	r8


; Have first Argument      (ha) in r21 r20 r19 r18
; may be +Inf , -Inf or a valid number (including special 0.000)
; Have the second argument (hb) in r25 r24 r23 r22
; Check if both signs are equal, then add else subtract

; unpack the exponent of b to r16 and mantissa of b to r20r19r18
	add	r20,r20	; Bit 7 = lowest bit of exponend b to carry
	mov	r8,r21 	; Bit 6...0 = Bit 7...1 of exponent b
	adc	r8,r8 	; r8 = exp b
.if denorm_support
	mov	r16,r8	
	or	r16,r18
	or	r16,r19
	or	r16,r20	; Then only if all four bytes are 0, then the value is 0 
.endif
				; Check if b = 0.00000
	j_eq	result_is_a_Check_a	; result is r25 r24 r23 r22
				; special case to correct 0.0 + 0.0
.if denorm_support
	or	r8,r8		; If the exp b
	brne	m23b_1
	; r8 = 0
	inc	r8		; correct exponent (r8=1, Carry=0)
	clc
	rjmp	m23b_0
m23b_1:
.endif
	sec			; Set Carry = M23
.if denorm_support
m23b_0:
.endif
	ror	r20		; r20 M22..M17

; r21 sign of b
; r16 8 bits exp b
; r20 r19 r18 mantissa of b (with bit 7 of r20=1)


; unpack the exponent of a to r17 and mantissa of a to r24r23r22
	add	r24,r24	; -> Carry
	mov	r17,r25 	; Bit 6...0 = Bit 7...1 of exponent a
	adc	r17,r17	; r17 = exp a

; r21 sign of b
; r16 8 bits exp b
; r20 r19 r18 mantissa of b (with bit 7 of r24=1)
.if denorm_support
	mov	r16,r17	
	or	r16,r22
	or	r16,r23
	or	r16,r24	; Then only if all four bytes are 0, then the value is 0 
.endif
				; Check if a = 0.00000


	j_eq	result_is_b_but_shift_ ;



.if denorm_support
	or	r17,r17		; If the exp a
	brne	m23a_1
	; r17 = 0
	inc	r17		; correct exponent (r17=1, Carry=0)
	clc
	rjmp	m23a_0
m23a_1:
.endif
	sec			; Set Carry = M23
.if denorm_support
m23a_0:
.endif
	ror	r24		; r24 Mb22..Mb17 a

	cpi	r17,0xff	; if EXP a == FF
	breq	NaN_or_Inf_a
	mov	r16,r8
	cpi	r16,0xff	; if EXP b == FF
	breq	NaN_or_Inf_b
	rjmp	both_normal


NaN_or_Inf_a:	; r17 = FF Check if it is NaN or +-Inf

	mov	r16,r24
	andi	r16,0x7F
	or	r16,r23
	or	r16,r22
	j_ne	return_nan
; a is +/-Inf

	mov	r16,r8
	cpi	r16,0xff		; if EXP b == FF
	j_ne	overflow		; a is +-Inf, b is a valid number
	mov	r16,r20
	andi	r16,0x7F
	or	r16,r19
	or	r16,r18
	brne	return_nan		; a is +/-Inf and b is NaN
; a is +/-Inf and b is +/-inf

	mov	r16,r21
	eor	r16,r25
	brmi	return_nan		; a is +/-Inf and b is -/+Inf
	
	rjmp	overflow		; sign is r25

	
NaN_or_Inf_b:
	mov	r16,r20
	andi	r16,0x7F
	or	r16,r19
	or	r16,r18
	mov	r25,r21
	j_eq	overflow	; sign is r25

return_nan:
	ldi	r22,0
	ldi	r23,0
	ldi	r24,0xC0
	ldi	r25,0x7F
; r25 sign of result
; r17 8 bits exp a of result
; r24 r23 r22 mantissa of result (with bit 7 of r24=1)
	rjmp	addresult



; r25 sign of a (and parts of the old exponent)
; r17 8 bits exp a
; r24 r23 r22 mantissa of a (with bit 7 of r24=1)

; Check if both exponents are same
both_normal:
	mov	r16,r17	; exp a
	sub	r16,r8 	; exp a - exp b : carry = 0 if a >= b
	brcc	no_swapab	; exp a >= exp b
; exp a < exp b
; swap a with b
	movw	r0,r22	; save Mb to old Mb
	movw	r22,r18	; Mb = Ma
	movw	r18,r0	; Ma = old Mb
	movw	r0,r24
	movw	r24,r20
	movw	r20,r0

; Swap exponents
	mov	r16,r8	; save old exp b
	mov	r8,r17	; exp b = old exp a
	mov	r17,r16	; exp b = old exp a

;	mov	r16,r17	; exp a
	sub	r16,r8 	; exp a - exp b : carry = 0 if a >= b
no_swapab:


; need to shift mantissa of b, i.e. r0r21r20
; r8 = exponent difference , exp a - exp b
shift_b:
; case 5 + 3
	clr	r15
	clr	r14		; extension 2nd byte
	cpi	r16,32	; Will shift more than mantissa precision ?
	brcs	add_shiftb24
; restore a, b is too small to add
; r25 sign of result
; r17 8 bits exp a of result
; r24 r23 r22 mantissa of result (with bit 7 of r24=1)
	rjmp	add_assemble

add_shiftb24:
	cpi	r16,24		; exp b - exp a, compare to 24
	brcs	add_shiftb16
	subi	r16,24
; perform 24 bit shift
.if round
	mov	r14,r19
	mov	r15,r20		; needed for smart rounding
.endif
	clr	r18
	clr	r19
	clr	r20
; adc r24,r1 not needed, becaue r23 = 0
	rjmp	add_shiftb_bits

add_shiftb16:
	cpi	r16,16		; exp b - exp a, compare to 16
	brcs	add_shiftb8
	subi	r16,16
; perform 16 bit shift
.if round
	mov	r14,r18
	mov	r15,r19		; needed for smart rounding
.endif
	mov	r18,r20
	clr	r19
	clr	r20
; adc r24,r1 not needed, becaue r23 = 0
	rjmp	add_shiftb_bits

add_shiftb8:
	cpi	r16,8
	brcs	add_shiftb_bits
	subi	r16,8
.if round
	mov	r14,r15
	mov	r15,r18		; needed for smart rounding
.endif
	mov	r18,r19		; perform 8 bit shift
	mov	r19,r20
	clr	r20

; The fast byte shifting hase been berformed
; Do now 0 to 7 bit shifts
	
add_shiftb_bits:
	or	r16,r16
	breq	addb_m

add_shiftb_bits_no_test:
	lsr	r20			; shift mantissa right (3bytes, by 1 bit)
	ror	r19
	ror	r18
.if round
	ror	r15			; Mantissa extension for smart rounding
	ror	r14
.endif
	dec	r16			; exponent difference
	brne	add_shiftb_bits_no_test

	

addb_m:

.if round
	eor	r13,r13	; extension byte 1
	eor	r12,r12	; extension byte 2
.endif

; As we shifted, r8 is invalid, r17 is valid for both
; Now r25 = sign a, r17 = separated exponent of a, r24 r23 r22 r13 r12 is mantissa of a
; Now r21 = sign b, r17 = separated exponent of b, r20 r19 r18 r15 r14 is mantissa of b

; Decide if we had to add, or to subtract
	mov	r1,r21
	eor	r1,r25
	j_mi	subtract


; Perform addition, both signs are egual
.if round


	.global addadd		; for simatmel debugger
addadd:
	add	r12,r14
	adc	r13,r15		; We think we add 0 to the Mantissa extension
	adc	r22,r18		; Add Mb to Ma, (Mb has been shifted right)
	adc	r23,r19
	adc	r24,r20
	brcc	add_noshift		; In the shifted mantissa this is an overflow
	ror	r24
	ror	r23
	ror	r22
	ror	r13			; r24 r32 r22 r13 = Ma, shifted 1 bit right, start 0 1 x x x 
	inc	r17
	breq	overflow
add_noshift:
	rjmp	add_round
add_noshift2:
;	inc	r17			; cannot shift back - instead increment the exponent
;	breq	overflow
add_round:
	cpi	r17,0xff
	breq	overflow
	clr	r1
	ldi	r16,0x80
	cp	r13,r16
	breq	toward_even
add_round_1:
	add	r13,r13
	adc	r22,r1
	adc	r23,r1
	adc	r24,r1
	brcc	add_round_done
	sec
	clr	r13
	lsr	r24
	ror	r23
	ror	r22
	ror	r13
	inc	r17
	rjmp	add_round
toward_even:
	bst	r22,0	; bit 0 of r22 to T
	bld	r13,7	; T to bit 7 or r13
	rjmp	add_round_1
	
add_round_done:



.else				; when we want to calculate without rounding
	add	r22,r18
	adc	r23,r19
	adc	r24,r20
	brcc	add_noshift3
	ror	r24
	ror	r23
	ror	r22
	inc	r17			; cannot shift back - instead increment the exponent
	breq	overflow
add_noshift3:
.endif

.if denorm_support
add_norm_a:
	or	r24,r24
	brmi	add_norm_3
	cpi	r17,1
	breq	add_norm_2	; Must be = 1
	or	r17,r17
	breq	add_norm_3

add_norm_2:
	dec	r17
add_norm_3:
.else
	or	r24,r24
	brpl	addresult0
.endif
add_assemble:
	.global addass	; for simatmel debugger
addass:

	add	r24,r24
	adc	r25,r25	; old sign to carry
	mov	r25,r17	; exp a
	ror	r25
	ror	r24
	rjmp	result_is_a

overflow:
	adc	r25,r25	; old sign to carry
	ldi	r25,0xfe	; Exponent = 0xFF
	ror	r25		; Same sign as input, but overflow, +Inf
	ldi	r24,0x80	; e0, M22..M16 HJH20070221
	ldi	r23,0	; M15..M8
	ldi	r22,0	; M7..M0
	rjmp	addresult
result_is_a_Check_a:
	mov	r16,r25
	andi	r16,0x7f
	or	r16,r24
	or	r16,r23
	or	r16,r22
	brne	result_is_a
; a = 0.0 b=0.0
	and	r25,r21
	andi	r25,0x80
	ldi	r24,0
	ldi	r23,0
	ldi	r22,0
	rjmp	addresult
	
addresult0:
	ldi	r25,0	; sign = 0 , Exponent = 0x00
	ldi	r24,0	; e0, M22..M16
;	ldi	r23,0	; M15..M8
;	ldi	r22,0	; M7..M0
	movw	r22,r24

result_is_a:	; result is r25r24r23r22
addresult:

	clr	r1	; GNU C convention
	pop	r8
	pop	r12
	pop	r14
	pop	r13
	pop	r15
	pop	r16
	pop	r17
	pop	r18
	pop	r19
	pop	r20
	pop	r21
	.global addret	; for simatmel debugger
addret:

	ret
; result is r25r24r23r22


result_is_b_but_shift_:
	movw	r22,r18	; M15...M0
	movw	r24,r20
	mov	r17,r8
	rjmp	add_norm_a





; The signs are different, need to subtract
	.global	subtract	; for simatmel debugger
subtract:
; As we shifted, r8 is invalid, r17 is valid for both
; Now r25 = sign a, r17 = separated exponent of a, r24 r23 r22 r13 r12 is mantissa of a
; Now r21 = sign b, r17 = separated exponent of b, r20 r19 r18 r15 r14 is mantissa of b

; decide which value is greater, consider 0.9001 > 0.90009
	cpc	r12,r14
	cpc	r13,r15
	cpc	r22,r18	; compare mantissa a against mantissa b
	cpc	r23,r19
	cpc	r24,r20
	brcc	noswap_sub
	; Carry = Clear, if a >= b
; Swap a <=> b
	movw	r0,r12
	movw	r12,r14
	movw	r14,r0
	movw	r0,r22	; save Mb to old Mb
	movw	r22,r18	; Mb = Ma
	movw	r18,r0	; Ma = old Mb
	movw	r0,r24
	movw	r24,r20
	movw	r20,r0
	.global	noswap_sub	; for simatmel debugger
noswap_sub:
; r17 is still the one and only valid exponent


.if round
	sub	r12,r14
	sbc	r13,r15
	sbc	r22,r18
.else
	sub	r22,r18
.endif
	sbc	r23,r19
	sbc	r24,r20
	breq	addresult0		; Result is zero

; The result is not zero
; we shift until the highest bit in r24 is set

; shift result left, until highest bit is set
	.global	sub_normb1	; for simatmel debugger
sub_normb1:
	or	r24,r24
	brne	sub_normb2
; Shift only if exponent is not below 8
	cpi	r17,9
.if denorm_support
	brcs	sub_normb2			; must not shift, exponent is zero
.else
	brcs	addresult0
.endif
	mov	r24,r23
	mov	r23,r22
.if round
	mov	r22,r13
	eor	r13,r13
.else
	ldi	r22,0
.endif
	subi	r17,8		; exp b (b was greater)
	rjmp	sub_normb1

; Some bit shifting until the highest bit is 1 (bit 7 in r24 = 1)
; or until the exponent a is decrementet to 0
	.global	sub_normb2	; for simatmel debugger
sub_normb2:
	or	r24,r24
	j_mi	add_round	; done, mantissa is in norm form
	or	r17,r17			; exp b
.if denorm_support
	j_eq	add_round	; We support denormalized values
.else
	j_eq	addresult0	; underflow
.endif

	.global	sub_normb3	; for simatmel debugger
sub_normb3:

	cpi	r17,1			; exp b
.if denorm_support
	j_eq	add_round	; We support denormalized values
.else
	j_eq	addresult0	; underflow
.endif
	dec	r17
.if round
	add	r13,r13
	adc	r22,r22
.else
	add	r22,r22
.endif
	adc	r23,r23
	adc	r24,r24
	brpl	sub_normb3
	rjmp	add_round
