
; .include "c:/home/cc/include/longbr.inc"
.include "../../../include/longbr.inc"

; Author : HJHerbert (2007-02-07)
;
; float fa = -17.444;
; float fb = 306.2;
; ...
; hr1 = fa / fb ;
;
; requires 515 cycles, = 32.1875 usec at 6 MHz ;
; The original library routine needed 1453 cycles, 90.75 usec
;
; Arithmetic Errors: less than 0.3 ppm of result
;
; used rs
; r22 - a mantissa 7..0
; r23 - a mantissa 15..8
; r24 - a exp 0 mantissa 22..16
; r25 - a sign, exp 7..1
; r18 - b mantissa 7..0
; r19 - b mantissa 15..8
; r20 - b exp 0 mantissa 22..16
; r21 - b sign, exp 7..1
; r26 - exponent of a
; r28 - exponent of b
; r27:r26 - Exponent difference, exp a - exp b + 0x7E
; r7 - mantissa 7..0 of result (during division)
; r8 - mantissa 15..8 of result (during division)
; r9 - mantissa 23..16 of result (during division)

; r1 = 0 

	round = 1 			; disable (0) or enable (1) rounding
	nan_support = 1		; disable (0) or enable (1) nan input support
	denorm_support = 1	;
	negative_0_support = 1	

; for binary compatibility with Intel and the PC need to set
; round = 1, nan_support = 1, denorm_support = 1, negative_0_support = 1
;
; for fastest operation can set all = 0
; Then the caller is responsible for the checking

	.data

	.text

; hr = ha / hb
; a r25 r24 r23 r22
; b r21 r20 r25 r18
; r = a / b
; Output :
; r25 r24 r23 r22 = a The result, (IEEE)

 	.global __divsf3   ; ffl ffl::operator*( ffl other )
 __divsf3:

	push	r6
	push	r7
	push	r8
	push	r9

	push	r18
	push	r19
	push	r20
	push	r21

	push	r26
	push	r27
	push	r28

; Have first Argument      (a) in r25 r24 r23 r22
; may be +Inf , -Inf or a valid number (including special 0.000)
; Have the second argument (b) in r21 r20 r19 r18

; unpack the exponent of b to r28 and mantissa of b to r20 r19 r18
	add	r20,r20	; Bit 7 = lowest bit of exponend b to carry
	mov	r28,r21 	; Bit 6...0 = Bit 7...1 of exponent b
	adc	r28,r28 	; r28 = exp b


.if denorm_support
	or	r28,r28		; Check if the exp b is 0
	brne	m23b_1
	; r28 = 0
	inc	r28		; correct exponent (r28=1, Carry=0)
	clc
	rjmp	m23b_0
m23b_1:
.endif
	sec			; Set Carry = M23
.if denorm_support
m23b_0:
.endif
	ror	r20		; r20 M22..M17

; r21 sign of b
; r27 8 bits exp b
; r20 r19 r18 mantissa of b (with bit 7 of r20=1)

; unpack the exponent of a to r26 and mantissa of a to r24r23r22
	add	r24,r24	; -> Carry
	mov	r26,r25 	; Bit 6...0 = Bit 7...1 of exponent a
	adc	r26,r26	; r26 = exp a

; r21 sign of b
; r27 8 bits exp b
; r20 r19 r18 mantissa of b (with bit 7 of r24=1)


.if denorm_support
	or	r26,r26		; If the exp a
	brne	m23a_1
	; r26 = 0
	inc	r26		; correct exponent (r26=1, Carry=0)
	clc
	rjmp	m23a_0
m23a_1:
.endif
	sec			; Set Carry = M23
.if denorm_support
m23a_0:
.endif
	ror	r24		; r24 Mb22..Mb17 a

	cpi	r26,0xff	; if EXP a == FF
	breq	NaN_or_Inf_a
	cpi	r28,0xff	; if EXP b == FF
	breq	NaN_or_Inf_b
	rjmp	both_normal


NaN_or_Inf_a:	; r26 = FF Check if it is NaN or +-Inf
; exp a is FF, a is either Inf or NaN
	mov	r27,r24
	andi	r27,0x7F
	or	r27,r23
	or	r27,r22
	brne	return_nan		; a = NaN
; a is +/-Inf

	cpi	r28,0xff		; if EXP b == FF
	breq	return_nan ; +-Inf / +-Inf = NaN  or +-INf / NaN = NaN

; a is +/-Inf and b is any number, or 0 - return +/-inf

;	mov	r27,r21
;	eor	r27,r25
;	brmi	return_nan		; a is +/-Inf and b is -/+Inf
	
joverflow:
	rjmp	overflow		; sign is r25 ^ r21

	
NaN_or_Inf_b:
; exp b is FF, exp a is not FF
	mov	r27,r20		; Check if b is Nan
	andi	r27,0x7F
	or	r27,r19
	or	r27,r18
	brne	return_nan
; b is +/- Inf
	mov	r27,r22		; M a
	or	r27,r23
	or	r27,r24
	breq	return_nan		; 0 * +/-Inf = NaN

	rjmp	overflow		; sign is r25 ^ r21

return_nan:
	ldi	r22,0
	ldi	r23,0
	ldi	r24,0xC0
	ldi	r25,0x7F
; r25 sign of result
; r26 8 bits exp a of result
; r24 r23 r22 mantissa of result (with bit 7 of r24=1)
	rjmp	divreturn		; Has result (NaN) 7FC00000 in r25:r24:r23:r22



; r25 sign of a (and parts of the old exponent)
; r26 8 bits exp a
; r24 r23 r22 mantissa of a (with bit 7 of r24=1)


both_normal:

	
; calculate the exponent
; r26 = exp a
; r28 = exp b
; r1 = 0 (Compiler convention)
	eor	r27,r27		; Result exponent extension, exponent of result = r27 r26
	sub	r26,r28		; Subtract exponents 1...254 + 1..254 = 2..508
					; from now, r28 in not more used as exponent
	sbc	r27,r1		; extension r27:r26 = 9 bit exponent

	ldi	r28,0x7e	; add defined offset 0x7E = 254
	add	r26,r28		;
	adc	r27,r1
	
	ldi	r28,8		; +8 because result is byte shifted
	add	r26,r28		; 
	adc	r27,r1		; r1=0 r27:r26 is the exponent

; Check if need to shift from subnormals
.if denorm_support
	or	r20,r20
	brne	bn20
	or	r19,r19
	brne	bn19
	adiw	r26,16		; multiply result by 65536
	mov	r20,r18			; multiply significant b by 65536 (this will divide the result by 65536)
	eor	r19,r19
	eor	r18,r18
	rjmp	bn20
bn19:
	adiw	r26,8		; multiply result by 256
	mov	r20,r19			; multiply significant b by 256 (this will divide the result by 256)
	mov	r19,r18
	eor	r18,r18
bn20:

	or	r24,r24
	brne	bn24
	or	r23,r23
	brne	bn23
	sbiw	r26,16		; divide result by 65536
	mov	r24,r22			; multiply significant a by 65536 (this will multiply the result by 65536)
	eor	r23,r23
	eor	r22,r22
	rjmp	bn24
bn23:
	sbiw	r26,8		; divide result by 256
	mov	r24,r23			; multiply significant a by 256 (this will multiply the result by 256)
	mov	r23,r22
	eor	r22,r22
bn24:


.endif


; Check if need to normalize the significant b
	or	r20,r20			; significant b
	breq	overflow	; divide by zero
	brmi	bm9
bm3:
	adiw	r26,1		; increment exp of result (this will double the result)
	add	r18,r18			; multiply significant b by 2 (this will half the result)
	adc	r19,r19
	adc	r20,r20
	brpl	bm3
bm9:

; Check if need to normalize the significant a
	or	r24,r24			; significant b
	breq	overflow	; divide by zero
	brmi	am9
am3:
	sbiw	r26,1		; decrement exp of result (this will half the result)
	add	r22,r22			; multiply significant a by 2 (this will double the result)
	adc	r23,r23
	adc	r24,r24
	brpl	am3
am9:

 	.global __divf3s_divide
__divf3s_divide:


; r9:r8:r7:r6 = Result
; r24:r23:r22 = significant a
; r20:r19:r18 = significant b
	eor	r6,r6			; clear result
	eor	r7,r7
	eor	r8,r8
	eor	r9,r9
	ldi	r28,25			; Bit count - loop over 24 bits
div_compare:
;; perform this for r28 = 24 ... r28 = 1
	cp	r22,r18			; Compare a with b
	cpc	r23,r19
	cpc	r24,r20
	brcs	nosub		; Do not perform subtraction, because a < b
subtract :
	sub	r22,r18	; a - b
	sbc	r23,r19
	sbc	r24,r20
	sec		
	adc	r6,r6
	adc	r7,r7		; shift result left, shift 1 into lsb
	adc	r8,r8
	adc	r9,r9
	rjmp	shifta
nosub:
	add	r6,r6
	adc	r7,r7		; shift result left
	adc	r8,r8
	adc	r9,r9
shifta:
	dec	r28		; The shift and binary digit counter
	breq	mantissaok

	add	r22,r22
	adc	r23,r23
	adc	r24,r24
	brcc	div_compare	; check before subtract
	; Carry is set :Has shifted 1 therefore next subtract will be valid
	rjmp	subtract

underflow:
	ldi	r22,0
	ldi	r23,0
.if negative_0_support
	eor	r25,r21
	andi	r25,0x80
	mov	r24,r22
.else
	movw	r24,r22		; there is only one zero and it is zero
.endif
	rjmp	divreturn

overflow:
	eor	r21,r25		; sign = sign ^ sign a
	add	r21,r21		; sign of result to carry

	ldi	r22,0
	ldi	r23,0
	ldi	r24,0x80		; exp0 = 1, Significant=0
	ldi	r25,0x7f<<1		; exp7..exp1
	ror	r25			; sign of result from carry to bit 7
; The result after + overflow is r21 r16 r21 r20 = 0x7F800000
; The result after - overflow is r21 r16 r21 r20 = 0xFF800000

; r25 = s, exp7..exp1
; r24 = exp0, M22..M16
; r23 = M15..M8
; r22 = M7..M0
	rjmp	divreturn

mantissaok:

 	.global __divsf3_done
__divsf3_done:

; new mantissa is in r9 r8 r7 r6 (32 bits)
; new exponent is still in r27:r26
	rcall	__normsf3
	
divreturn:
; r25 = s, exp7..exp1
; r24 = exp0, M22..M16
; r23 = M15..M8
; r22 = M7..M0

	pop	r28
	pop	r27
	pop	r26

	pop	r21
	pop	r20
	pop	r19
	pop	r18

	pop	r9
	pop	r8
	pop	r7
	pop	r6

	eor	r1,r1			; C convention r8 = 0 

	ret
