
; .include "c:/home/cc/include/longbr.inc"
.include "../../../include/longbr.inc"

; Author : HJHerbert (2007-02-07)
; HJHerbert 2007-04-17
;
; Input
; r9 r8 r7 r6 significant (32 Bits, r6 used for rounding)
; r27 r26     Exponent with Offset
; r25,bit7 sign a
; r21,bit7 sign b
; Output :
; r25 r24 r25 r24 = a The result, (IEEE)
;
; used by __mulsf3 and by __divsf3
;

	round = 1 			; disable (0) or enable (1) rounding
	nan_support = 1		; disable (0) or enable (1) nan input support
	denorm_support = 1	;
	negative_0_support = 1	

;
	.data

	.text
	
 	.global __normsf3
__normsf3:
	
normsf3:
; The mantissa is in normal form, if the highest bit is set (between 0.5 and 0.999999940395355224609375)
.if denorm_support

	mov	r18,r6
	or	r18,r7
	or	r18,r8
	or	r18,r9
	j_eq	underflow

; Shift left until the highest bit is in 7,r9
noleft8m:
	or	r9,r9
	brne	noleft8z
	mov	r9,r8
	mov	r8,r7
	mov	r7,r6
	eor	r6,r6		; The significant looks alike having been multiplied by 256
	sbiw	r26,8		; exponent -8 : is alike number divided by 256
	rjmp	noleft8m
noleft8z:
noleft1m:
	or	r9,r9
	brmi	noleft1z
	add	r6,r6
	adc	r7,r7
	adc	r8,r8
	adc	r9,r9
	sbiw	r26,1
	rjmp	noleft1m

j_underflow:
	rjmp	underflow
j_overflow:
	rjmp	overflow


noleft1z:

	; Check if r27 r26 <= 0
	cpi	r26,1
	cpc	r27,r1	; r1=0
	brge	doround

; while the exponent is below 1, shift right, 
; because we have denormed number
noright1m:
	lsr	r9
	ror	r8
	ror	r7
	ror	r6
	adiw	r26,1
	cpi	r26,1
	cpc	r27,r1	; r1=0
	brlt	noright1m
;	rjmp	doround
	
.else
	cpi	r26,1
	cpc	r27,r1	; r27:r26 - 1 
	brlt	j_underflow	; no denorm_support
.endif


doround:

;	The result mantissa is in r9 r8 r7 r6
; Check if we want to round up, round down or round toward even

	ldi	r22,0x80	; round up or cut
	cp	r6,r22		
	brne	mulresult1
;round_even:
	; from 3.5 make 4.0
	; from 4.5 make 4.0
	sbrs	r7,0	; skip if bit in register is set
	rjmp	mulresult2	; - is even,keep it
;	rjmp	mulresult1	; - round it up

mulresult1:
	add	r6,r6
	eor	r6,r6
	adc	r7,r1
	adc	r8,r1
	adc	r9,r1
	brcc	mulresult2
; Rounding caused mantissa overflow
	ror	r9
	ror	r8
	ror	r7
	adiw	r26,1
	cpi	r26,0xff	; check if r27:r26 >= 0x00ff
	brcc	overflow
	rjmp	mulresult2

	
mulresult2:
; r9:r8:r7 is the significant

; Check if the exponent is >= 0xFF
	cpi	r26,0xff
	cpc	r27,r1
	brcc	overflow	; r27:r26 >= 0xff

.if denorm_support
	cpi	r26,1
	brne	mulresult3
	or	r9,r9
	brmi	mulresult3
	eor	r26,r26	; denormed value
mulresult3:
.else		; no denorm_support
	or	r9,r9
	brpl	underflow
	cpi	r26,0
	breq	underflow
.endif

; Assemble the result from sign, r26, r9:r8:r7
	eor	r21,r25	; sign of result = sign of a ^ sign of b (bit 7)
; r21 is sign of result
	mov	r22,r7
	mov	r23,r8
	mov	r24,r9
	add	r24,r24	; unused highest bit M23 is always 1, no information
	mov	r25,r26	; full exponent, exp7..exp0
	add	r21,r21	; sign of result to carry
	ror	r25		; sign, exp7...exp1
	ror	r24		; exp0, M22..M16
	ret


underflow:
	ldi	r22,0
	ldi	r23,0
.if negative_0_support
	eor	r25,r21
	andi	r25,0x80
	mov	r24,r22
.else
	movw	r24,r22
.endif
	ret
	
overflow:
	eor	r25,r21	; sign = sign ^ sign b
	ldi	r22,0
	ldi	r23,0
	ldi	r24,0x80
	ori	r25,0x7f	; exp7..exp1
	ret
