
#include <heport.h>
#include <helq.h>
#include <hegettim.h>
#include <heimath.h>
#include <getlam.h>

// This source is very hardware depend
// This source is to be used by TC
// AVR : Use up to eight channels at PORTA of atmega32 (10 bit) six channel at PORTC of atmega8
// __TURBOC__ Use the ADDA14 ADC Card

#ifdef AVR
#include <tshar.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#endif


#ifdef _MSC_VER
#include <heserial.h>			// prototypes for all functions defined here
#endif

#ifdef __TURBOC__
#define ADC_ENABLE_ADDA14 0
#define ADC_ENABLE_LAM002 0
#define ADC_ENABLE_LPT1 1 // Digital Input 0x379: 5 channels: Pin 15,13,12,10,11
#define ADC_ENABLE_LPT2 0 // Digital Input 0x279: 5 channels: Pin 15,13,12,10,11

#define MIN_CHANNEL_ADDA14	 0
#define MAX_CHANNEL_ADDA14	 (ADC_ENABLE_ADDA14*16+MIN_CHANNEL_ADDA14)

#define MIN_CHANNEL_LAM002	 (MAX_CHANNEL_ADDA14)
#define MAX_CHANNEL_LAM002	 (ADC_ENABLE_LAM002*8+MIN_CHANNEL_LAM002)

#define MIN_CHANNEL_LPT1	 (MAX_CHANNEL_LAM002)
#define MAX_CHANNEL_LPT1	 (ADC_ENABLE_LPT1*5+MIN_CHANNEL_LPT1)

#define MIN_CHANNEL_LPT2	 (MAX_CHANNEL_LPT1)
#define MAX_CHANNEL_LPT2	 (ADC_ENABLE_LPT2*5+MIN_CHANNEL_LPT2)


#define MAX_ADC_CHANNELS	 MAX_CHANNEL_LPT2

#define portadr 0x360					// original 170-1f0 modified 360-3e0
										// see home/html/hjh/pc/adda14.htm

#define LAMVAL_MAXAGE 800L				// Maximum Age 800 Milliseconds

// Checking on HAMDUS:
// MUXDELAY CONVDELAY
// 35000	3000	  10 mV Battery
// 25000	2000	  14 mV Battery
// 25000	800		  12 mV Battery
// 25000	400		  14 mV Battery
// 25000	150		  14 mV Battery
//	5000	150		  16 mV Battery
//	8000	500		  16 mV Battery
//	3000	 50		  16 mV Battery
//	   1	  1		  12 mV Battery
#define MUXDELAY  3000L
#define CONVDELAY 1L



char txLamPage[LAMPAGECHAR] ;

FUNCT sword adc(         				// __TURBOC__
			int channel )
			// channel 0 to 15 : ADDA-14 Card (ISA) in HAMDUS
			// channel 16 to 20 : ADDA-14 Card (ISA) in HAMDUS
{

#if ADC_ENABLE_ADDA14
if ( channel >= MIN_CHANNEL_ADDA14 && channel < MAX_CHANNEL_ADDA14 )	// ADDA-14
	{
	sword val ;


	uword code ;		// read word
	int i ;
	uchar codeh ;
	uchar codel ;
	disable();				// interrupts
//	outportb( portadr, channel );
//	outportb( portadr+1, 0 );
	outport( portadr, channel );
	dlnsec( MUXDELAY );		// Mux-Delay
	for ( i = 0 ; i < 8 ; ++i )		// 8 Bits low convert
		{
		inportb( portadr+8 );	// convert next bit
		dlnsec( CONVDELAY );
		}
	for ( i = 0 ; i < 8 ; ++i )		// 8 Bits high convert
		{
		inportb( portadr+12 );	// convert next bit
		dlnsec( CONVDELAY );
		}
	codel = inportb( portadr+2 );		// Get 16 bit Value
	codeh = ( inportb( portadr+3 ) & 0x3f ) - 0x20 ;		// Get 16 bit Value
	code = ( codeh << 8 ) | codel ;
	val = code * 4 ;			// val always -32768 .. 32767
	enable();			// Interrupts
	return val ;
	}
#endif // ADC_ENABLE_ADDA14

#if ADC_ENABLE_LPT1
if ( channel >= MIN_CHANNEL_LPT1 && channel < MAX_CHANNEL_LPT1 )	// LPT1
	{
	if ( channel == MIN_CHANNEL_LPT1 ) return inportb(0x379) & 0x08 ? 1 : 0 ; // LPT1 Pin 15
	if ( channel == MIN_CHANNEL_LPT1+1 ) return inportb(0x379) & 0x10 ? 1 : 0 ; // LPT1 Pin 13
	if ( channel == MIN_CHANNEL_LPT1+2 ) return inportb(0x379) & 0x20 ? 1 : 0 ; // LPT1 Pin 12
	if ( channel == MIN_CHANNEL_LPT1+3 ) return inportb(0x379) & 0x40 ? 1 : 0 ; // LPT1 Pin 10
	if ( channel == MIN_CHANNEL_LPT1+4 ) return inportb(0x379) & 0x80 ? 0 : 1 ; // LPT1 Pin 11
	}
#endif


#if ADC_ENABLE_LPT2
if ( channel >= MIN_CHANNEL_LPT2 && channel < MAX_CHANNEL_LPT2 )	// LPT2
	{
	if ( channel == MIN_CHANNEL_LPT2 ) return inportb(0x279) & 0x08 ? 1 : 0 ; // LPT2 Pin 15
	if ( channel == MIN_CHANNEL_LPT2+1 ) return inportb(0x279) & 0x10 ? 1 : 0 ; // LPT2 Pin 13
	if ( channel == MIN_CHANNEL_LPT2+2 ) return inportb(0x279) & 0x20 ? 1 : 0 ; // LPT2 Pin 12
	if ( channel == MIN_CHANNEL_LPT2+3 ) return inportb(0x279) & 0x40 ? 1 : 0 ; // LPT2 Pin 10
	if ( channel == MIN_CHANNEL_LPT2+4 ) return inportb(0x279) & 0x80 ? 0 : 1 ; // LPT2 Pin 11
	}
#endif

#if ADC_ENABLE_LAM002
if ( channel >= MIN_CHANNEL_LAM002 && channel < MAX_CHANNEL_LAM002 )
	{
	sword val ;
	if ( GetLam( txLamPage, channel-MIN_CHANNEL_LAM002, &val ) )
		{
		uprintf( "Cannot find Index %d Channel %d in LAM Page", channel-MIN_CHANNEL_LAM002, channel-MIN_CHANNEL_LAM002 +1 );
		return 0 ;
		}
	return val ;
	}
#endif			// ADC_ENABLE_LAM002


return 0 ;
}


// Now this the millivolts to adc conversion is fix.
// In future this may be calibrated throuh a /home/ini/dac.ini file
FUNCT sword adc_to_mvolt( int channel, sword adcval )	// __TURBOC__
{

#if ADC_ENABLE_ADDA14
if ( channel >= MIN_CHANNEL_ADDA14 && channel < MAX_CHANNEL_ADDA14 )	// ADDA-14
	{
	long interval ;

	interval = 8500L * adcval ;
	return (sword)(interval / 32000L) ;
	}
#endif

return adcval ; // undefined output channel - no hardware present
}



FUNCT sword mvolt_to_adc( int channel, sword millivoltvalue )	// __TURBOC__
{

#if ADC_ENABLE_ADDA14
if ( channel >= MIN_CHANNEL_ADDA14 && channel < MAX_CHANNEL_ADDA14 )	// ADDA-14
	{
	long interval ;

	interval = millivoltvalue * 32000L ;
	return (sword)(interval / 8500L) ;
	}
#endif

return millivoltvalue ;
 ;	// undefined output channel - no hardware present
}





FUNCT int nadc()				// __TURBOC__
{
return MAX_ADC_CHANNELS ;		// count of adc converters in system
}



FUNCT int adcopen()				// __TURBOC__
{
#if ADC_ENABLE_ADDA14
caldlnsec();
#endif


return 0 ;		// All ok
}
#endif		// endif TC



#ifdef _MSC_VER
sword	raw[8] ;


FUNCT int adcopen(void)
{
int rv ;
static struct sserial vserial ;

vserial.baud = 38400 ;				 // as integer number
vserial.stop = 0 ;				// 0 or 1
vserial.parity = 'n' ;				// 'n' 'e' or 'o'
vserial.data = 8 ;				// 5, 6, 7, 8
vserial.handshake = 'n' ;

serset( 1, vserial );
rv = seropen(1);

return 0 ;

}





FUNCT sword adc( int channel )
// Get the values from a memory array, Just for checking of programs which are desoigned to run on a AVR
{


#if 0
uchar lowbyte ;
uchar highbyte ;
uchar cr ;

while( serisicha(1) ) serichaw( 1 );
serocha( 1, 'A'-0x40 );
serichaw(1);
serocha( 1, channel );
highbyte = serichaw( 1 );
lowbyte = serichaw( 1 );
cr = serichaw( 1 );
return ( highbyte << 8 ) | lowbyte ;
#endif

char buffer[8] ;
const char *pb ;
sword value ;
int i ;
uchar cr ;

while( serisicha(1) ) serichaw( 1 );
serocha( 1, 'n' );
serocha( 1, '0' );
serocha( 1, '0' + channel );
serocha( 1, 0x0d );
for(;;)
	{
	cr = serichaw(1);
	if ( cr == '=' ) break ;
	}
i = 0 ;
for(;;)
	{
	if ( i >= sizeof(buffer)-1 ) break ;
	cr = serichaw(1);
	if ( cr == 0x0d ) break ;
	if ( cr == 0x0a ) break ;
	buffer[i++] = cr ;
	}
pb = buffer ;
asctosw( &pb, &value );
return value ;
}



FUNCT int sendport( uchar port, uchar value )
{
char buffer[8] ;

nsprintf( buffer, sizeof(buffer), "p%02X,%02X", port, value );

while( serisicha(1) ) serichaw( 1 );
serputs( 1, buffer );
serocha( 1, 0x0d );
return 0 ;
}


FUNCT sword adc_to_mvolt( int channel, sword adcval )
{

return (sword) (mul_sw_sw( adcval , 2560 * 2 ) >> 16 );
}


FUNCT sword mvolt_to_adc( int channel, sword mv )
{
return mv ;
}


#endif


// Start the ADC of an AVR e.g. ATMega8535 or ATMega32
// Wait the conversion time
// and return the readout
#ifdef AVR

FUNCT int adcopen(void)
{
return 0 ;
}

FUNCT sword adc(
	int m )			// Channel number 0..7
					// According to ATMEGA32 data sheet
					// 0..7 = Single ended conversation
					// 8..9 = ADC0..1 against ADC0, Gain=10x
					// 10..11 = ADC0..1 against ADC0, Gain=200x
					// 12..13 = ADC2..3 against ADC2, Gain=10x
					// 14..15 = ADC2..3 against ADC2, Gain=200x
					// 16..23 = ADC0..7 against ADC1, Gain=1x
					// 24..29 = ADC0..5 against ADC2, Gain=1x
					// 30 = 1/2 VREF (1.22 V)
					// 31 = GND
{
ADMUX = 0xC0 |			// Internal 2.56 V reference
		1 << ADLAR ;	// left justified result
DDRA = 0 ;				// Input
PORTA = 0 ;				// No internal Pull-up
//		SFIOR = 0 ;
ADCSRA = 1 << ADEN	|	// ADC Enable
		 0 << ADSC	|	// Do not start conversion yet
		 0 << ADATE	 |	// conversion is not finished
		 0 << ADIF	|	//
		 0 << ADIE	|	// No Interrupt Enable
		 7 ;		// 3 prescaler bits: 128 , 16000000Hz / 128 = 125000
tsnext( "adc-pause");
ADMUX = (ADMUX & 0xE0) | m ;	// Single Ended Input ADC m
ADCSRA |=  1 << ADEN  | 1 << ADSC ;		// Start conversation
// Now wait until conversation done
while ( ( ADCSRA & ( 1 << ADSC ) ) )
	{
	tsnext( "adc" );
	}

if ( m <= 8 || m >= 30 )
	{		// Single ended
	return ADCW>>1 ;
	}
else
	{
	return ADCW ;
	}
}


FUNCT sword adc_to_mvolt( int channel, sword adcval )
{

return mul_sw_sw( adcval , 2560 * 2 ) >> 16 ;
}

#endif
