// getticks.cpp
// get the number of ticks elapsed as 32 bit value
// The ticks is the systems fastest possible timer
// PC / DOS : 4.77 MHz / 4 
// Windows2000 : 1 msec
// OSK : 1/100 Hz
// AVR : depending on the timer predivider, if it is 8, then F_CPU/8 (see ../include/hegettim.h)

#include <heport.h>
#include <helq.h>
#include <hegettim.h>

#if defined( AVR )
#include <avr/io.h>
#endif

#if defined( MSVC40 ) || defined( MDOS40 )
#include <mapidefs.h>	// FILETIME ...
#endif



#ifdef OSK
FUNCT	ulong	gettick_disabled( void ) /* get count of microseconds after midnight */
{
int atime ;
int adate ;
short aday ;
int atick ;	 /* Highbyte : tick per second , Lowbyte : current clock tick */
ulong tick256 ;

_sysdate( 3, &atime, &adate, &aday, &atick );
tick256 = ( atime * 256		   /* seconds after midnight */
			+ ( ( atick & 0x7FFF ) * 256 / ( ( atick >> 16 ) & 0x7FFF ) ) );
// return 1000000L * tick256 / 256L ;	  /* Milliseconds after midnight */
return 15625L * ( tick256 % (256L*60L) ) / 64L ;	 /* Milliseconds after midnight */
}
#endif // OSK




#if defined( __TURBOC__ )

FUNCT	uword	getticks_disabled( void ) /* get count of microseconds after midnight */
{
uchar ctrlow , ctrhigh ;	// content of 16 bit counter
uword ctr ;

outp( 0x43, 0 );	// Latch counter
ctrlow	= inp( 0x40 );
ctrhigh = inp( 0x40 );

ctr = ~ (( ctrhigh << 8 ) + ctrlow) ;

return ctr ;

}
#endif									// __TURBOC__



#if defined( AVR )


#if GETTICK_TIMER == 0

// Timer0
// 2^8 = 256
// 256 * 64 / 16.000000 MHz = 1024 usec
uchar lastcnt0 ;	// The last read value
uchar actcnt0 ;		// the actual read value
ulong countextender ;

// call first tickopen(), then use the timer gettick()

FUNCT ulong gettick( void )				// AVR
{

actcnt0 = TCNT0 ;		// Read the 16 bit value ( 2MHz ) from counter 1
if ( actcnt0 < lastcnt0 )
	{					// once per 65536 * 0.5 us --- once per 32768 usec
	countextender += 256 ;
	}
lastcnt0 = actcnt0 ;
return countextender | actcnt0 ;
}


FUNCT void getwordtick( uword &result )
{

actcnt0 = TCNT0 ;		// Read the 16 bit value ( 2MHz ) from counter 1
if ( actcnt0 < lastcnt0 )
	{					// once per 65536 * 0.5 us --- once per 32768 usec
	countextender += 256 ;
	}
lastcnt0 = actcnt0 ;
result = countextender | actcnt0 ;
}
#endif

#if GETTICK_TIMER == 1

// Timer1
// 2^16 = 65536
// 65536 * 8 / 16.000000 MHz = 32768 usec
uword lastcnt1 ;	// The last read value
uword actcnt1 ;		// the actual read value
uword countextender ;

// call first tickopen(), then use the timer gettick()

FUNCT ulong gettick( void )				// AVR
{
uchar oldstat ;
union {
	ulong ul ;
	struct {
		uword lw ;
		uword hw ;
		} w ;
	} retval ;
oldstat = SREG ;
SREG &= ~(1<<SREG_I) ;	// Clear the Interrupt Enable bit
actcnt1 = TCNT1 ;		// Read the 16 bit value ( 2MHz ) from counter 1
if ( actcnt1 < lastcnt1 )
	{					// once per 65536 * 0.5 us --- once per 32768 usec
	++countextender ;
	}
retval.w.lw = lastcnt1 = actcnt1 ;
retval.w.hw = countextender ;
SREG = oldstat ;
return retval.ul ;
}
#endif




#endif


// /////////////////////////////////////////////////////////////
