// This routine multiplies two 16 bit integers,
// The return value is a 32 bit integer
//
// See also mulswsw.cpp : Signed 16 bit * signed 16 bit 

#include <heport.h>
#include <heimath.h>		// definition of mul_uw_uw
#ifdef AVR
#include <avr/io.h>
#endif



#ifndef AVR
// Other processors do not need speed up
FUNCT ulong mul_uw_uw( uword x , uword y )
{
ulong res ;

res = (ulong)x * (ulong)y ;
return res ;
}

#else

#if 0 /* Software- Multiply */
// input:
// r23_r22 = x.h_x.l
// r25_r24 = y.h_y.l
// may destroy r18 r19 r20 r21 r22 r23 r24* r25* r26* r27*
// may not destroy r14 r15 r16 r17 r28=YL r29=YH
// return r25_r24_r23_r22


FUNCT ulong dummy_mul_uw_uw( uword x , uword y )
{

// asm(" .text " "\n");
asm(" ; " "\n");
asm("  .global mul_uw_uw" "\n");
asm(" ; See as well mul_uc_uc : 8 bit by 8 bit, result 16 bits " "\n");
asm(" ; " "\n");
asm(" ; Function mul_uw_uw " "\n");
asm(" ; multiply unsigned 16 bit by 16 bit " "\n");
asm(" ; " "\n");
asm(" ; Input: " "\n");
asm(" ;     r23:r22   : The factor " "\n");
asm(" ;     r25:r24   : The factor " "\n");
asm(" ; Results " "\n");
asm(" ;     r25:r24:r23:r22    : result " "\n");
asm(" ; Destroyed registers " "\n");
asm(" ;     r18 r19 r20 r21 r26 r27" "\n");
asm(" ; This program is verified by qqarith.asm " "\n");
asm(" ; " "\n");
//asm(" _Z9mul_uw_uwtt: " "\n");
asm("mul_uw_uw: " "\n");
asm(" 	push r16 ; " "\n");
asm(" ; Clear start values " "\n");
asm(" 	mov r26,r22 ; factor x to interim storage " "\n");
asm(" 	mov	r27,r23 " "\n");
asm(" 	mov r18,r24 ; factor y to interim storage " "\n");
asm(" 	mov	r19,r25 " "\n");
asm(" 	clr r20 " "\n");
asm(" 	clr r21 " "\n");
asm(" ; Clear result r25:r24;r23;r22 " "\n");
asm(" 	clr r22 " "\n");
asm(" 	clr r23 " "\n");
asm(" 	clr r24 " "\n");
asm(" 	clr r25 " "\n");
asm(" ; " "\n");
asm(" mul_uw_uw_3: " "\n");
asm(" 	lsr	r27 " "\n");
asm(" 	ror	r26		; next binary digit of right factor to carry " "\n");
asm(" 	brcc	mul_uw_uw_4 " "\n");
asm(" ; Add the (shifted) left factor into result " "\n");
asm(" 	add	r22,r18 " "\n");
asm(" 	adc	r23,r19 " "\n");
asm(" 	adc	r24,r20 ; add 0" "\n");
asm(" 	adc	r25,r21 ; add 0" "\n");
asm(" mul_uw_uw_4: " "\n");
asm(" ; shift the factor y by 1 bit to left" "\n");
asm(" 	add	r18,r18 " "\n");
asm(" 	adc	r19,r19 " "\n");
asm(" 	adc	r20,r20 " "\n");
asm(" 	adc	r21,r21 " "\n");
asm(" 	mov	r16,r26		; no more of right factor left " "\n");
asm(" 	or	r16,r27 " "\n");
asm(" 	brne mul_uw_uw_3 " "\n");
asm(" ; result is 32 bits in r25:r24:r23:r22 " "\n");
asm(" 	pop	r16 " "\n");
asm(" 	ret " "\n");

return 0 ;
}

#else

// Hardware multiply MUL
// input:
// r23_r22 = x.h_x.l
// r25_r24 = y.h_y.l
// may destroy r18 r19 r20 r21 r22 r23 r24* r25* r26* r27*
// may not destroy r14 r15 r16 r17 r28=YL r29=YH
// return r25_r24_r23_r22


FUNCT slong dummy_mul_uw_uw( sword x , sword y )
{
//asm(" .text " "\n");
asm(" .global mul_uw_uw" "\n");
asm(" ; See as well mul_uc_uc : 8 bit by 8 bit, result 16 bits " "\n");
asm(" ; " "\n");
asm(" ; Function mul_uw_uw " "\n");
asm(" ; multiply unsigned 16 bit by 16 bit " "\n");
asm(" ; " "\n");
asm(" ; Input: " "\n");
asm(" ;     r23:r22   : The factor " "\n");
asm(" ;     r25:r24   : The factor " "\n");
asm(" ; Results " "\n");
asm(" ;     r25:r24:r23:r22    : result" "\n");
asm(" ; Destroyed registers " "\n");
asm(" ;     r18 " "\n");
asm(" ; This program is verified by qqarith.asm " "\n");
asm(" ; " "\n");
//asm(" _Z9mul_uw_uwtt: " "\n");
asm("mul_uw_uw: " "\n");
asm(" 	push r0 ; " "\n");
asm(" 	push r1 ; " "\n");
asm(" 	push r16 ; " "\n");
asm(" ; Clear start values " "\n");
asm(" 	clr	r16 " "\n" );
asm(" 	mov r20,r22 ; factor x to interim storage " "\n");
asm(" 	mov	r21,r23 " "\n");
asm(" 	mov r18,r24 ; factor y to interim storage " "\n");
asm(" 	mov	r19,r25 " "\n");

asm(" 	mul	r20,r18 ; low byte * low byte" "\n" );
asm(" 	mov	r22,r0 ; low result word" "\n" );
asm(" 	mov	r23,r1 ; low result word" "\n" );
asm(" 	mul	r21,r19 ; high byte * high byte" "\n" );
asm(" 	mov	r24,r0 ; high result word" "\n" );
asm(" 	mov	r25,r1 ; high result word" "\n" );
asm(" 	mul	r20,r19 ; low byte * high byte" "\n" );
asm(" 	add	r23,r0 ; medium result word" "\n" );
asm(" 	adc	r24,r1 ; medium result word" "\n" );
asm(" 	adc	r25,r16 ; carry to highest byte" "\n" );
asm(" 	mul	r21,r18 ; low byte * high byte" "\n" );
asm(" 	add	r23,r0 ; medium result word" "\n" );
asm(" 	adc	r24,r1 ; medium result word" "\n" );
asm(" 	adc	r25,r16 ; carry to highest byte" "\n" );
asm(" ; result is 32 bits in r25:r24:r23:r22 " "\n");
asm(" 	pop	r16 " "\n");
asm(" 	pop	r1 " "\n");
asm(" 	pop	r0 " "\n");
asm(" 	ret " "\n");

return 0 ;
}

#endif

#endif
