/*
Here is a collection of programs which, under different operating
systems make possible to keep track on parallel programming.

  Each main program
*/

#include <heport.h>
#include <helq.h>
#include <tshar.h>
#include <tsser.h>
#include <tsinout.h>


#define TS_DEBUG



int tshar_task_no ;
char in_charbuffer[NPRGPATH][NIN_CHARBUFFER] ;



#if !defined AVR
const char *tsnext_p[NPRGPATH] ;	// tsnext string pointer
const char *proces_p[NPRGPATH] ;	// process name string pointer
int *spsave[NPRGPATH] ;
int *bpsave[NPRGPATH] ;

bool flag_defined[NPRGPATH] = { true } ;
#endif


#if defined AVR
uword frsave[NPRGPATH];			//
#endif

// Which character input / character output / the task tshar_task_no has to use
void (*(vout_char[NPRGPATH]))(const uchar c);
// Send a character to the terminal
// here for each task the output routine has to be set, e.g. the addresse of out_char or tslcd_ocha
// call just out_char to send a character

uchar ((*(vin_char[NPRGPATH]))(void));
// unsigned char (*(vin_char[NPRGPATH]))(void);
// here for each task the input routine has to be set, e.g. the addresse of in_char
// call just in_char to get a character

bool (*(vin_check[NPRGPATH]))(void);
// here for each task the input check routine has to be set, e.g. the addresse of in_check
// call just in_check to check if there is a character available



#if defined( LINUX )

#define NREGSAVE 4
FUNCT int tseinri(				// LINUX
	int index, 					// Task Number (1...NPRGPATH-1)
	void (*funaddr)(), 			// Tasks main function
	char *pname,				// Tasks name
	void *stackbase,				// Base of Tasks Stack
	int stacksize)				// Size of tasks stack
{
int *tos ;
int rv ;

if ( index >  0 && index < NPRGPATH )
	{
	proces_p[index] = pname ;
	tos = ((int *)stackbase) + ( ( stacksize / sizeof(int) ) & 0xfffffffc ) ;
	*--tos = (int)funaddr ; // retaddr
	tos -= NREGSAVE+1 ;			// 20 Bytes, 5 registers ; (ebp) eax ebx ecx edx
	spsave[index] = tos ;
	bpsave[index] = tos+NREGSAVE ;
	flag_defined[index] = true ;
	rv = 0 ;
	}
else
	{
	rv = 1 ;	// error number
	}
return rv ;
}
#endif


#if defined( LINUX )
void tsnext(const char *pt )
{
tsnext_p[ tshar_task_no ] = pt ;
// Save context of actual function
	asm("	pushl %eax");
	asm("	pushl %ebx");
	asm("	pushl %ecx");
	asm("	pushl %edx");		// the amount of saved registers here is NREGSAVE = 4
	asm("	movl tshar_task_no, %ebx");
	asm("	movl %esp,spsave(,%ebx,4)");
	asm("	movl %ebp,bpsave(,%ebx,4)");

	for(;;)
		{
		if ( ++tshar_task_no >= NPRGPATH ) tshar_task_no = 0 ;
		if ( flag_defined[ tshar_task_no ] ) break ;
		}

// get context of next function
	asm("	movl tshar_task_no,%ebx");
	asm("	movl spsave(,%ebx,4),%esp");
	asm("	movl bpsave(,%ebx,4),%ebp");
	asm("	popl %edx");
	asm("	popl %ecx");
	asm("	popl %ebx");
	asm("	popl %eax");
return ;
}
#endif




#if defined( MDOS50 ) || defined( MDOS40 )
FUNCT int tseinri(				// MDOS40 MDOS50
	int index, 					// Task Number (1...NPRGPATH-1)
	void (*funaddr)(), 			// Tasks main function
	char *pname,				// Tasks name
	void *stackbase,				// Base of Tasks Stack
	int stacksize)				// Size of tasks stack
{
int *tos ;
int rv ;

if ( index >= 0 && index < NPRGPATH )
	{
	proces_p[index] = pname ;
	tos = ((int *)stackbase) + ( ( stacksize / sizeof(int) ) & 0xfffffffc ) ;
	*--tos = (int)funaddr ; // retaddr
	tos -= 7 ;			// 28 Bytes, 7 registers ; (ebp,ebx,edi,esi) eax ecx edx
	spsave[index] = tos ;
	bpsave[index] = tos+6 ;
	flag_defined[index] = true ;
	rv = 0 ;
	}
else
	{
	rv = 1 ;	// error number
	}
return rv ;
}
#endif


#if defined( MDOS50 ) || defined( MDOS40 )
void tsnext(const char *pt )
{
tsnext_p[ tshar_task_no ] = pt ;
// Save context of actual function
__asm
	{
	push eax
	push ecx
	push edx
	mov ebx, tshar_task_no
	mov spsave[ebx*4], esp
	mov bpsave[ebx*4], ebp
	}

	for(;;)
		{
		if ( ++tshar_task_no >= NPRGPATH ) tshar_task_no = 0 ;
		if ( flag_defined[ tshar_task_no ] ) break ;
		}

// get context of next function
__asm
	{
	mov ebx, tshar_task_no
	mov esp, spsave[ebx*4]
	mov ebp, bpsave[ebx*4]
	pop edx
	pop ecx
	pop eax
	}
return ;
}
#endif









#if defined( MDOS10 ) || defined( MSVC10 )
FUNCT int tseinri(				// MDOS10 MSVC10
	int index, 					// Task Number (1...NPRGPATH-1)
	void (*funaddr)(), 			// Tasks main function
	char *pname,				// Tasks name
	void *stackbase,				// Base of Tasks Stack
	int stacksize)				// Size of tasks stack
{
int *tos ;
int rv ;

if ( index >= 0 && index < NPRGPATH )
	{
	proces_p[index] = pname ;
	tos = ((int *)stackbase) + ( ( stacksize / sizeof(int) ) & 0xfffffffc ) ;
	*--tos = (int)funaddr ; // retaddr
	tos -= 7 ;			// 14 Bytes, 7 registers ; (bp,bx,di,si) ax cx dx
	spsave[index] = tos ;
	bpsave[index] = tos+6 ;
	flag_defined[index] = true ;
	rv = 0 ;
	}
else
	{
	rv = 1 ;	// error number
	}
return rv ;
}
#endif






#if defined( MDOS10 ) || defined( MSVC10 )
void tsnext(const char *pt )
{  
tsnext_p[ tshar_task_no ] = pt ;
// Save context of actual function
__asm
	{
	push ax
	push cx
	push dx
	mov bx, tshar_task_no 
	add bx,bx
	mov spsave[bx], sp
	mov bpsave[bx], bp
	}

	for(;;)
		{
		if ( ++tshar_task_no >= NPRGPATH ) tshar_task_no = 0 ;
		if ( flag_defined[ tshar_task_no ] ) break ;
		}

// get context of next function
__asm
	{
	mov bx, tshar_task_no
	add	bx,bx
	mov sp, spsave[bx]
	mov bp, bpsave[bx]
	pop dx
	pop cx
	pop ax
	}
return ;
}
#endif




#if defined __TURBOC__


FUNCT int tseinri(int index, void (*funaddr)(), char *pname,
void *stackbase, int stacksize)
{
int *tos ;
int rv ;

if ( index >= 0 && index < NPRGPATH )
	{
	proces_p[index] = pname ;
	tos = ((int *)stackbase) + ( ( stacksize / sizeof(int) ) & 0xfffffffc ) ;
	*--tos = (int)funaddr ; // retaddr
	tos -= 3+1 ;			// 8 Bytes, 3 registers ; ax cx dx
	spsave[index] = tos ;
	bpsave[index] = tos+4 ;
	flag_defined[index] = true ;
	rv = 0 ;
	}
else
	{
	rv = 1 ;	// error number
	}
return rv ;
}


FUNCT void tsnext(const char *pt )		// __TURBOC__
{
tsnext_p[ tshar_task_no ] = pt ;
// Save context of actual function
asm {
	push ax
	push cx
	push dx
	mov bx, tshar_task_no
	add bx,bx
	mov spsave[bx], sp
	mov bpsave[bx], bp
	}

	for(;;)
		{
		if ( ++tshar_task_no >= NPRGPATH ) tshar_task_no = 0 ;
		if ( flag_defined[ tshar_task_no ] ) break ;
		}

// get context of next function
asm {
	mov bx, tshar_task_no
	add bx,bx
	mov sp, spsave[bx]
	mov bp, bpsave[bx]
	pop dx
	pop cx
	pop ax
	}
return ;
}
#endif	// __TURBOC__



// ******************************************************************* AVR
#if defined AVR
FUNCT void tseinri_P(			// AVR
	int index, 					// Task Number (1...NPRGPATH-1)
	void (*funaddr)(), 			// Tasks main function
	void *stackbase,				// Base of Tasks Stack
	int stacksize)				// Size of tasks stack
{
uword *pf ;
uchar *pfc ;
// pf[0] = 0 ;		// unused

// frsave[0] = (uword)pf ;
pf = (uword *) ((char *)stackbase + (stacksize - (2*2) ));
pf[0] = 0x3344 ; // (uword)stackbase + stacksize ; // preset caller frame

pfc = (uchar *)(pf + 1);			// Higher address = return addresse
*pfc++ = (uchar)((uword)funaddr>>8) ;
*pfc++ = (uchar)(uword)funaddr ;

pf = (uword *) ((char *)stackbase + (stacksize - (2*2) -1 -20));
frsave[index] = ((uword) pf) ;

return ;
}


// 2006-05-23
// three emtpy loops of one main program and two functions, NPRGPATH = 3
// takes 278 Cycles, At 16 MHz = 17.375 us


FUNCT void dummy_tsnext_P(void)		// AVR
{

// Save old Frame pointer (callers caller stack)
//		PUSH	R28				 Push register on stack
//		PUSH	R29				 Push register on stack
//
//		IN		R28,0x3D		 In from SPL
//		IN		R29,0x3E		 In from SPH
//		SBIW	R28,0x02		 Subtract immediate from word, needs 2 bytes in tsnext
//		IN		R0,0x3F			 In from I/O location
//		CLI						 Global Interrupt Disable
//		OUT		0x3E,R29		 Out to I/O location
//		OUT		0x3F,R0			 Out to I/O location interrup enable after next statement
//		OUT		0x3D,R28		 Out to I/O location
//		STD		Y+2,R25			 Store indirect with displacement , save *pt
//		STD		Y+1,R24			 Store indirect with displacement

// here
asm("	.text                      \n" );
asm("	.global tsnext_P                      \n" );
asm("tsnext_P:                       \n" );
asm("	.global _Z8tsnext_Pv                      \n" );
asm("_Z8tsnext_Pv:                       \n" );
asm("	push	r2                      \n" );	// 1
asm("	push	r3                      \n" );	// 2
asm("	push	r4                      \n" );	// 3
asm("	push	r5                      \n" );	// 4
asm("	push	r6                      \n" );	// 5
asm("	push	r7                      \n" );	// 6
asm("	push	r8                      \n" );	// 7
asm("	push	r9                      \n" );	// 8
asm("	push	r10                     \n" );	// 9
asm("	push	r11                     \n" );	// 10
asm("	push	r12                     \n" );	// 11
asm("	push	r13                     \n" );	// 12
asm("	push	r14                     \n" );	// 13
asm("	push	r15                     \n" );	// 14
asm("	push	r16                     \n" );	// 15
asm("	push	r17                     \n" );	// 16
asm("	push	r24                     \n" );	// 17
asm("	push	r25                     \n" );	// 18
asm("	push	r28                     \n" );	// 19
asm("	push	r29                     \n" );	// 20
asm("	push	r30                     \n" );	// 21
asm("	push	r31                     \n" );	// 22

asm("	in	r28,0x3D                     \n" );		// SPL
asm("	in	r29,0x3E                     \n" );		// SPH
asm("	lds	r24,tshar_task_no                   \n" );
asm("	add	r24,r24                      \n" );
asm("	sub	r25,r25                      \n" );
asm("	ldi	r30,lo8(frsave)              \n" );
asm("	ldi	r31,hi8(frsave)              \n" );	// The Stack pointer arraz
asm("	add	r30,r24                      \n" );
asm("	adc	r31,r25                      \n" ); // Z = &frsave[tshar_task_no]
asm("	std	Z+0,r28                      \n" );
asm("	std	Z+1,r29                      \n" );		// The frame is saved
asm("tsnext_P_1:                         \n" );
asm("	lds	r24,tshar_task_no                   \n" );
asm("	inc	r24                          \n" );
asm("	sts	tshar_task_no,r24                   \n" );
asm("	cpi	r24, %0                      \n" : : "M" (NPRGPATH) : );		// NPRGPATH
//asm("	cpi	r24,4                        \n" );		// NPRGPATH
asm("	brlo	tsnext_P_2               \n" );
asm("	clr	r24                          \n" );
asm("	sts	tshar_task_no,r24                   \n" );
asm("tsnext_P_2:                         \n" );
//asm("	lds	r24,tshar_task_no                   \n" );
asm("	add	r24,r24                      \n" );
asm("	sub	r25,r25                      \n" );
asm("	ldi	r30,lo8(frsave)                      \n" );
asm("	ldi	r31,hi8(frsave)                      \n" );
asm("	add	r30,r24                      \n" );
asm("	adc	r31,r25                      \n" );
asm("	ldd	r28,Z+0                      \n" );
asm("	ldd	r29,Z+1                      \n" );		// The frame is restored
asm("	or	r28,r29                      \n" );
asm("	breq	tsnext_P_1                      \n" );
// frsave[tshar_task_no] = pf ; // Pointer to frame
// SP = pf
// *pf	   = pf[0] = local
// *(pf+1) = pf[1] = local
// *(pf+2) = pf[2] = Callers r29 YH
// *(pf+3) = pf[3] = Callers r28 YL
// *(pf+4) = pf[4] = Callers PC
// *(pf+5) = pf[5] = Callers PC

asm("	lds	r24,tshar_task_no                      \n" );
asm("	add	r24,r24                      \n" );
asm("	sub	r25,r25                      \n" );
asm("	ldi	r30,lo8(frsave)                      \n" );
asm("	ldi	r31,hi8(frsave)                      \n" );
asm("	add	r30,r24                      \n" );
asm("	adc	r31,r25                      \n" );

asm("	ldd	r28,Z+0                      \n" );
asm("	ldd	r29,Z+1                      \n" );		// The stack pointer is restored
// asm("	adiw	r28,2                      \n" );

// Now SP points to some place (the stack of this function)
// SP+1=L SP+2=H : the caller frame pointer
// SP+3=L SP+4=H : the return addresse (start addresse in first loop)
asm("	in	r0,0x3f                      \n" );
asm("	cli                      \n" );
asm("	out	0x3E,r29                      \n" );
asm("	out	0x3F,r0                      \n" );
asm("	out	0x3D,r28                      \n" );
asm("	pop	r31                      \n" );		// ZL
asm("	pop	r30                      \n" );		// ZH
asm("	pop	r29                      \n" );		// YL
asm("	pop	r28                      \n" );		// YH Y = Frame Pointer
asm("	pop	r25                      \n" );
asm("	pop	r24                      \n" );
asm(" 	pop r17                      \n" );
asm(" 	pop r16                      \n" );
asm(" 	pop r15                      \n" );
asm(" 	pop r14                      \n" );
asm(" 	pop r13                      \n" );
asm(" 	pop r12                      \n" );
asm(" 	pop r11                      \n" );
asm(" 	pop r10                      \n" );
asm(" 	pop r9                      \n" );
asm(" 	pop r8                      \n" );
asm(" 	pop r7                      \n" );
asm(" 	pop r6                      \n" );
asm(" 	pop r5                      \n" );
asm(" 	pop r4                      \n" );
asm(" 	pop r3                      \n" );
asm(" 	pop r2                      \n" );
asm("	ret                      \n" );

//		   ADIW	   R28,0x02			Add immediate to word
//		   IN	   R0,0x3F			In from I/O location
//		   CLI						Global Interrupt Disable
//		   OUT	   0x3E,R29			Out to I/O location // restore SPH
//		   OUT	   0x3F,R0			Out to I/O location
//		   OUT	   0x3D,R28			Out to I/O location // restore SPL
//		   POP	   R29				Pop register from stack // REstore callers frame
//		   POP	   R28				Pop register from stack
//		   RET						Subroutine return

}

#endif		// ifdef AVR

FUNCT void tsopen(void)
{
int i ;

for ( i = 0 ; i < NPRGPATH ; ++i )
	{
	vout_char[i] = tsser_out_char ;
	vin_char[i] = tsser_in_char ;
	vin_check[i] = tsser_in_check ;
	}
return ;
}

FUNCT void out_char( const uchar c )
{
if ( vout_char[tshar_task_no] )
	{
	vout_char[tshar_task_no](c);	// call the preset function
	}
}

FUNCT uchar in_char( void )
{
if ( vin_char[tshar_task_no] )
	{
	return vin_char[tshar_task_no]();	// call the preset function
	}
return 0 ;
}

FUNCT bool in_check( void )
{
if ( vin_char[tshar_task_no] )
	{
	return vin_check[tshar_task_no]();	// call the preset function
	}
return false ;
}


