#include <heport.h>
#include <hegettim.h>
#include <tshar.h>
#include <tsinout.h>
#include <hevt.h>


// 2007-07-07
// __TURBOC__ : Support 2 Lines x 234 characters
//              Delay times nominal, as delaytick is repaired
// #define CP 0x80 // Bit 7

// Settings for a Toshiba TLC-691 (1x24)
// Even if that device is 1x24, it internally has 2x32, just the display
// window is smaller. From the second line only some part is visible.
//
// This driver is tested only for a Toshiba TLC-691 1 row by 24
// 
// This driver can be used with a PC and Borlands __TURBOC__
// or can be used with avr-gcc for AVR controllers
// When use PORTC, then disable the JTAG flag

#define LCD_YMAX 2	// one or two lines
#define LCD_XMAX 24	// columns

// Select the hardware connection of the LCD by type

#ifdef __TURBOC__		// Use the PC Printer Port
// Verified HJH20070805

#define DPORT 0x378		// The printer Port Address
#define INPORT 0x378		// read back

#define ENABLE 0x08 // Bit 3
#define DATAINSTR  0x04 // Bit 2
// #define READWRITE 0x04 // Bit 2

#define DATAMASK	0xF0	// Bit 7 ... bit 4 = 4 data lines

// #define outportb(ap,data) ap = data
// #define inportb(ap) ap
#define cbi(port,mask) outportb( port, d &= ~(mask) )
#define sbi(port,mask) outportb( port, d |=  (mask) )

// #define cbi(port,mask) outportb( port, inportb(port) & ~(mask) )
// #define sbi(port,mask) outportb( port, inportb(port) |  (mask) )

#endif				// __TURBOC__

#ifdef AVR
#define ENABLE 0x08 // Bit 3
#define DATAINSTR  0x04 // Bit 2
// #define READWRITE 0x04 // = GND, = Write, never read anything back

#define DATAMASK	0xF0	// Data Bits
#define DPORT PORTD			// Today with this project use PORT D tsaccu:D
#define INPORT PIND
#define DDRPORT DDRD
// Warning : Atmega32 needs JTAG disabled (in the fuses, use onyprog) 
// Else only bits 0 1 . . . . 6 7 of PORTC are usable

#define outportb(ap,data) ap = data
#define inportb(ap) ap

//#define cbi(port,mask) port = ( d &= ~(mask) )
//#define sbi(port,mask) port = ( d |=   mask  )
#define cbi(port,mask) port &= ~(mask) 
#define sbi(port,mask) port |=   mask  

#endif

uchar d ;				// actually given to Data Port




#ifdef DPORT			// Only ifdef __TURBOC__ or ifdef AVR


FUNCT void tslcd_help( void )
{
#ifdef __TURBOC__
out_puts( "Connect the 14 Pins of the L2432:          \n" );
out_puts( "LCD: Printer Port      : Signal            \n" );
out_puts( " 1 : 18,19,20,21,22,23 : GND               \n" );
out_puts( " 2 : none              : +5Volts           \n" );
out_puts( " 3 : 2 D0              : unused            \n" );
out_puts( " 4 : 3 D1              : unused            \n" );
out_puts( " 5 : 4 D2    0x04      : Data=1/Instr=0    \n" );
out_puts( " 6 : 5 D3    0x08      : Enable 1-> 0      \n" );
out_puts( " 7 : -                 : D0 not used       \n" );
out_puts( " 8 : -                 : D1 not used       \n" );
out_puts( " 9 : -                 : D2 not used       \n" );
out_puts( "10 : -                 : D3 not used       \n" );
out_puts( "11 : 6 D4              : D4                \n" );
out_puts( "12 : 7 D5              : D5                \n" );
out_puts( "13 : 8 D6              : D6                \n" );
out_puts( "14 : 9 D7              : D7" );
#endif

#ifdef AVR_HELP
out_puts( "Connect the 14 Pins of the L2432:\n" );
out_puts( "LCD: ATmega32    : Signal\n" );
out_puts( " 1 :  9          : GND\n" );
out_puts( " 2 : 10          : +5Volts\n" );
out_puts( " 3 : 22 PC0      : \n" );
out_puts( " 4 : 23 PC1      : \n" );
out_puts( " 5 : 24 PC2 0x04 : Data=1/Instr=0\n" );
out_puts( " 6 : 25 PC3 0x08 : Enable 1-> 0\n" );
out_puts( " 7 : 11 GND      : D0 not used\n" );
out_puts( " 8 : 11 GND      : D1 not used\n" );
out_puts( " 9 : 11 GND      : D2 not used\n" );
out_puts( "10 : 11 GND      : D3 not used\n" );
out_puts( "11 : 26 PC4      : D4\n" );
out_puts( "12 : 27 PC5      : D5\n" );
out_puts( "13 : 28 PC6      : D6\n" );
out_puts( "14 : 29 PC7      : D7" );
#endif
}




FUNCT void enable_pulse(void)
{
cbi( DPORT, ENABLE );
//delaytick( msectotick(0.002) );

sbi( DPORT, ENABLE );
//delaytick( msectotick(0.005) );
}



FUNCT void tslcd_code1( uchar code )	// output from he high nibble
{
cbi( DPORT, DATAINSTR );   // Clear Bit in Interface: Want to send instruction

cbi( DPORT, DATAMASK );		// Clear all Data bits
sbi( DPORT, code & DATAMASK );
enable_pulse();
// delaytick( msectotick(0.3) );
}



FUNCT void tslcd_code( uchar code )		// Send a Code Byte to the LCD2432
{
// sbi( DPORT, DATAINSTR );   // Clear Bit in Interface: Want to send instruction
tslcd_code1( code );
tslcd_code1( code<< 4 );


delaytick( msectotick(0.04) );
}



FUNCT void tslcd_ocha( const uchar z )
{
if ( z )
	{
	cbi( DPORT, DATAMASK );
	sbi( DPORT, DATAINSTR | ((z<<0) & 0xf0) );   	// Set Bit in Interface
//	d &= 0x0F ;
//	d |= (z<<0) & 0xf0 ;
//	outportb( DPORT, d );	// Output data high nibble


	enable_pulse();

	// delaytick( msectotick(0.01) );

	cbi( DPORT, DATAMASK );
	sbi( DPORT, ((z<<4) & 0xf0) );   	// Set Bit in Interface
//	d &= 0x0F ;
//	d |= (z<<4) & 0xf0 ;
//	outportb( DPORT, d );	// Output low high nibble


	enable_pulse();

	delaytick( msectotick(0.04) );	// HJH070707
	// not enough 0.01
	//	   enough 0.04
	// ... according to /home/html/etronik/lcd_display/lcdl2432-howtocontrol.htm
	}
}




// The Seiko LCD 2432 requires this init after each power off/on
FUNCT int tslcd_init( void )
{
uchar i ;

#ifdef AVR
DDRPORT = 0xFC;		// 6 bits are output Bit 7 ... Bit 2
#endif


// Check the interface
#ifdef AVR_00
DPORT = 0xFC ;
delaytick( msectotick(1.64) );
if ( ( INPORT & 0xFC) != 0xFC ) return 1 ;		// Port Error
DPORT = 0x00 ;
delaytick( msectotick(1.64) );
if ( ( INPORT & 0xFC) != 0x00 ) return 2 ;		// Port Error
#endif

#ifdef AVR
delaytick( msectotick(2000) );		// Wait until the LCD is started
#endif


// cbi( DPORT, READWRITE );       // = Write
sbi( DPORT, ENABLE );

cbi( DPORT, DATAINSTR );   // Clear Bit in Interface - want to send instruction, not data


tslcd_code1( 0x20 );			// 4 Bit mode
delaytick( msectotick(0.04) );
tslcd_code1( 0x20 );			// 4 Bit mode
delaytick( msectotick(0.04) );
tslcd_code1( 0x20 );			// 4 Bit mode
delaytick( msectotick(0.04) );

for( i = 2+1 ; --i ; )
	{

	tslcd_code( 0x20			// Set function
				+ 0 * 0x10		// Data Lenght : 0=4 Bit 1=8 bit
				+ 1 * 0x08		// 2 Display Lines
				+ 0 * 0x04 );	// Character Font

	tslcd_code( 0x01 ) ;		// Clear Display 1.64 ms
	delaytick( msectotick(2.64) );
	}

tslcd_code( 0x04
			+ 0x02			// Cursor Direction
							// +0x02 : left to right (normal)
							// 0x00 : right to left
			+0x00			// +0x01 shift display
							// 0x00 move cursor
					 );
tslcd_code( 0x08 			// Switch On :
			+ 1 * 0x04 		// Display on ( Not neccesary ?! )
			+ 1 * 0x02 );	// Cursor On

tslcd_code( 0x18 );	// Set Shift Mode

tslcd_home();
return 0 ;
}



FUNCT void tslcd_home(void)
{
tslcd_pos( 0, 0 );
}



FUNCT void tslcd_pos(uchar x, uchar y)
{

if ( y >= LCD_YMAX ) y = 0 ;
if ( x >= LCD_XMAX ) x = 0 ;

tslcd_code( 0x40	// set CGRAM - Addresse
			+ 0x1   );	// to 24 (one line length) (0.04 msec)
#if LCD_YMAX == 1
tslcd_code( 0x80	// set DDRAM - Adresse     (0.04 msec)
			+ x +1 );
#endif


#if LCD_YMAX == 2
tslcd_code( 0x80	// set DDRAM - Adresse     (0.04 msec)
			+ x +1 + (y?64:0) );
#endif


#if LCD_YMAX == 4
#	if LCD_XMAX == 16
tslcd_code( 0x80	// set DDRAM - Adresse     (0.04 msec)
			+ x +1 + ((y&1)?64:0) + ((y&2)?16:0  );
#	else
tslcd_code( 0x80	// set DDRAM - Adresse     (0.04 msec)
			+ x +1 + ((y&1)?64:0) + ((y&2)?20:0  );
#	endif
#endif

}
// 0x40+1   0x80+x+1 -- But only if the history is good



FUNCT void tslcd_cls(void)
{
int i ;
int y ;

for( y = 0 ; y < LCD_YMAX ; ++y )
	{
	tslcd_pos(0,y);
	for( i = LCD_XMAX+1 ; --i ; )
		{
		tslcd_ocha(' ');
		}
	}
}



FUNCT void tslcd_puts( const char *p )
{
while( *p )
	{
	tslcd_ocha( *p++ );
	}
}

#else	// ifndef DPORT

// The case where we have no LCD, Not __TURBOC__ , nor AVR
// Then we do fill an array "lcd_display" which can be examined the MSVC50 - Debugger

char lcd_display[LCD_YMAX][LCD_XMAX+1] ;
uint lcd_x ;
uint lcd_y ;



FUNCT void tslcd_cls(void )
{
uint x ;
uint y ;

for ( x = 0 ; x < LCD_XMAX ; ++x )
	{
	for ( y = 0 ; y < LCD_YMAX ; ++y )
		{
		lcd_display[y][x] = ' ';
		}
	}
return ;
}



FUNCT void tslcd_help(void )
{
return ;
}



FUNCT int tslcd_init(void )
{
lcd_x = 0 ;
lcd_y = 0 ;
return 0 ;
}



FUNCT void tslcd_ocha(uchar c )
{
if ( c ) 
	{
	if ( lcd_x < LCD_XMAX && lcd_y < LCD_YMAX )
		{
		lcd_display[lcd_y][lcd_x] = c ;
		++lcd_x ;
		}
	}
return ;
}



FUNCT void tslcd_pos(uchar x, uchar y)
{
lcd_x = x ;
lcd_y = y ;
return ;
}

#endif		// DPORT

