// From J. Herberts library LQ

#define DB_INFO 0		// activation of DBprintf(( ... )); logging of operation
#define DB_ENABLED 0	// activation of debugging logging of occured faults

#include <heport.h>		// special type names, uword, sword ...
#include <helq.h>		// uprintf
#include <tshar.h>
#include <tsinout.h>		// prototypes for all functions defined here



#if defined(MDOS40) || defined(MSVC40) || defined(MDOS10) || defined(MSVC10)
	// When MDOS40, then select if vt or serial
// #define MDOS40_COM1
#define USE_VT
#endif

#ifdef LINUX	// On Linux Use vt or serial
#define USE_VT
#endif


#ifdef __TURBOC__
#include <dos.h>
// #define TURBOC_PORT
#define USE_VT
#endif


#if defined( USE_VT )
#include <hevt.h>
#endif



// ---------------------------------------- AVR begin
#ifdef AVR


#ifndef UCSRB
#define UCSRA UCSR0A
#define UCSRB UCSR0B
#define UDR UDR0
#define UBRRL UBRR0L
#define UBRRH UBRR0H
#endif

FUNCT void tsseropen( void )		// AVR
// initalize the serial Port
{



#define UBRRinitval (((F_CPU+8L*BAUD)/(16L*BAUD))-1L) // See also /home/atmel/baudrate.xls

UCSRB = 0<<RXCIE |
		0<<TXCIE |
		0<<UDRIE |
		1<<RXEN |
		1<<TXEN |
		0<<UCSZ2 |
		0<<RXB8 |
		0<<TXB8 ;

#ifdef UCSRC
/*UCSRC= 1<<URSEL |	// Select UCSRC instead UBRHH
	   1<<USBS |	// 1 = 2 Stop bits
	   3<<UCSZ0 ;	// UCSZ2=0 UCSZ1=1 UCSZ0=1 = 8 bits
*/
#endif

if ( UBRRinitval/256 )
	{
	UBRRH = (uchar)(UBRRinitval/256) ;
	}
UBRRL = (uchar)UBRRinitval ;

//	; 7	  RXCIE Receive Interrupt enable
//	; 6	  TXCIE Transmit Interrupt enable
//	; 5	  UDRIE Data register empty interrupt enable
//	; 4 * RXEN	Receiver Enable
//	; 3 * TXEN	Transmitter Enable
//	; 2	  CHR9	nine data bits
//	; 1	  RXB8	the controller puts here the received 8th data bit
//	; 0	  TXB8	Transmit data bit 8,
//	ldi r16,0x18
//	out UART_CONTROL,r16	; UCR=0x0a / 0x2A


return ;
}



FUNCT void tsserclose( void )		// AVR
{
UCSRB = 0<<RXCIE |
		0<<TXCIE |
		0<<UDRIE |
		0<<RXEN |
		0<<TXEN |
	//	0<<CHR9 |
		0<<RXB8 |
		0<<TXB8 ;
return ;
}



FUNCT bool tsser_in_check( void )		// AVR
{
if ( UCSRA & ( 1<<RXC) )
	{
	return true ;
	}
else
	{
	return false ;
	}
}



FUNCT uchar tsser_in_char()			// AVR
{

while ( tsser_in_check() == false )
	{
	tsnext( "tsser_in_char" );
	}
return UDR ;
}


FUNCT void tsser_out_char( const uchar tc )	   // AVR
{
// Register UCSRA :
// 7
do
	{
	tsnext( "tsser_out_char" );
	}
	while ( !(UCSRA & (1<<UDRE)) ) ;
UDR = tc ;
return ;
}
#endif
// ---------------------------------------- AVR end




















// ---------------------------------------- MDOS40_COM1 begin

#ifdef MDOS40_COM1
#include <mapidefs.h>
HANDLE fdtsser ;

FUNCT void tsseropen( void )		// MDOS40_COM1
{
SECURITY_ATTRIBUTES seca ;
COMMTIMEOUTS  CommTimeOuts ;

seca.nLength = sizeof(SECURITY_ATTRIBUTES);
seca.lpSecurityDescriptor = 0 ; // use systems default
seca.bInheritHandle = FALSE ;	// do not inherit handle

fdtsser = CreateFile(
	"COM1",			  // #1 LPCTSTR	 lpFileName,
		//										  // pointer to name of the file
	GENERIC_READ | GENERIC_WRITE,	// #2DWORD	dwDesiredAccess,
		//										  // access (read-write) mode
	FILE_SHARE_READ | FILE_SHARE_WRITE ,
	&seca,							// #4 no security attrs
	OPEN_EXISTING,					// #5
		FILE_ATTRIBUTE_NORMAL |
		// FILE_FLAG_OVERLAPPED |		// overlapped I/O
		0 ,								// end of parameter #6
	NULL );							// parameter #7

if ( fdtsser == INVALID_HANDLE_VALUE )
	{
	uprintf( "cannot open %s for RW", "COM1" );
	return	;
	}


SetCommMask( fdtsser, EV_RXCHAR ) ;


// setup device buffers
SetupComm( fdtsser, 4096, 4096 ) ;


// purge any information in the buffer
PurgeComm( fdtsser, PURGE_TXABORT | PURGE_RXABORT |
								  PURGE_TXCLEAR | PURGE_RXCLEAR ) ;

// set up for overlapped I/O

CommTimeOuts.ReadIntervalTimeout = 0xFFFFFFFF ;
CommTimeOuts.ReadTotalTimeoutMultiplier = 0 ;
CommTimeOuts.ReadTotalTimeoutConstant = 1000 ;
CommTimeOuts.WriteTotalTimeoutMultiplier = 0 ;
CommTimeOuts.WriteTotalTimeoutConstant = 1000 ;
SetCommTimeouts( fdtsser, &CommTimeOuts ) ;

return ;	  // opened succesful
}



FUNCT void tsserclose( void )		// MDOS40_COM1
{
CloseHandle( fdtsser );
return ;
}



FUNCT bool tsser_in_check( void )		// MDOS40_COM1
{
bool flag_result ;

COMSTAT	   ComStat ;
DWORD	   dwErrorFlags;

ClearCommError( fdtsser, &dwErrorFlags, &ComStat ) ;
if ( dwErrorFlags )
	{
	if ( dwErrorFlags & CE_BREAK )
		{
		Eprintf(( "CE_BREAK" ));
		}
	if ( dwErrorFlags & CE_DNS)
		{
		Eprintf(( "CE_DNS" ));
		}
	if ( dwErrorFlags & CE_FRAME )
		{
		Eprintf(( "CE_FRAME" ));
		}
	if ( dwErrorFlags & CE_IOE )
		{
		Eprintf(( "CE_IOE" ));
		}
	if ( dwErrorFlags & CE_MODE )
		{
		Eprintf(( "CE_MODE" ));
		}
	if ( dwErrorFlags & CE_OOP )
		{
		Eprintf(( "CE_OOP" ));
		}
	if ( dwErrorFlags & CE_OVERRUN )
		{
		Eprintf(( "CE_OVERRUN" ));
		}
	if ( dwErrorFlags & CE_PTO )
		{
		Eprintf(( "CE_PTO" ));
		}
	if ( dwErrorFlags & CE_RXOVER )
		{
		Eprintf(( "CE_RXOVER" ));
		}
	if ( dwErrorFlags & CE_RXPARITY )
		{
		Eprintf(( "CE_RXPARITY" ));
		}
	if ( dwErrorFlags & CE_TXFULL )
		{
		Eprintf(( "CE_TXFULL" ));
		}
	}
if ( ComStat.cbInQue )
	{		// at least one character for ReadFile available
	flag_result = true ;
	}
else
	{
	flag_result = false ;
	}
return flag_result ;
}



FUNCT uchar tsser_in_char()			// MDOS40_COM1
{
char c ;
BOOL fReadStat ;
DWORD dwLength ;

while ( tsser_in_check() == false )
	{
	tsnext( "tsser_in_char" );
	}
// there is a byte available at input - read it
fReadStat = ReadFile(		// returns BOOL
		fdtsser,
		&c,			   // lpBuffer
		(DWORD)1,				// NumberOfBytesToRead
		&dwLength,		// lpNumberOfBytesRead
		NULL );			// The lpOverlapped parameter is not supported
return c ;
}


FUNCT void tsser_out_char( const uchar c )	   // MDOS40_COM1
{
unsigned long dwBytesWritten ;
BOOL		fWriteStat ;


fWriteStat = WriteFile( fdtsser, &c, 1,
					   &dwBytesWritten, NULL ) ;
return ;
}
#endif
// ---------------------------------------- MDOS40_COM1 end







// ---------------------------------------- USE_VT begin
#ifdef USE_VT
FUNCT void tsseropen( void )		// USE_VT
{
vtopen();
return ;
}



FUNCT void tsserclose( void )		// USE_VT
{
vtclose();
return ;
}



FUNCT bool tsser_in_check( void )		// USE_VT
{
return vtiskey() ;		// Check the PC Keyboard
}



FUNCT uchar tsser_in_char()			// USE_VT
{
while ( tsser_in_check() == false )
	{
	tsnext( "tsser_in_char" );
	}
return vtkey();		// read one character from PC keyboard
}



FUNCT void tsser_out_char( const uchar c )	   // USE_VT
{
vtocha( c );
return ;
}
#endif
// ---------------------------------------- USE_VT end

