// qqtsfloa - Demonstrates the dialog through a serial (simple) interface
// This main program demonstrates the usage of floating point routines for the ATMEL
// together with the functions for serial interface tsser.cpp
//
// Checked on LINUX (i386), MDOS50
// For Atmel : Linux, WinAVR
// 
// Link together with the programs in ~/home/cc/lq/  ffladd.s ffldiv.s and fflmul.s
//
// On PC in qqtstrm, after enter [F6]t
//

#include <math.h>
#include <heport.h>
#include <helq.h>
#include <hemath.h>
#include <heimath.h>
#include <tshar.h>
#include <hevt.h>
#include <hegettim.h>
#include <tsinout.h>
#include <tsser.h>

#ifdef AVR
#include <avr/io.h>
#endif

#ifdef AVR
#define NSTK1 120
#else
#define NSTK1 2000
#endif






// The test cases
#define TESTCASE_ADD_1
#define TESTCASE_MUL_1
#define TESTCASE_DIV_1



void *stk1[NSTK1] ;		// 00
#define STACK_EMPTY 0x7EAB	// for Stackcheck - command z

uword counter ;
#ifdef AVR
		// The stack must not go over a 0x100 page limit ?
#if ( stk1 & 0xff00 ) != ( stk1+NSTK1 & 0xff00 )
#error "stk1 over page (256) boundary"
#endif
#if ( stk2 & 0xff00 ) != ( stk2+NSTK2 & 0xff00 )
#error "stk2 over page (256) boundary"
#endif
#endif

extern struct sdate_uc date_uc ;
//	  ulong day ;
//	  uchar hour ;
//	  uchar minute ;
//	  uchar second ;


// For timing tests I need to have these external.
// Then the compiler and optimizer think tsnext() 
// will change them and generates code
float ha ;	
float hb ;	
float hr1 ;
float hr2 ;
float hr3 ;
float hr4 ;
float hr5 ;	
float hr6 ;	
float hr7 ;	
uchar uba ;
uword uwa ;
ulong ula ;




// Output all 4 bytes of a float as hexadecimal (without losses) serial
FUNCT void ohex8( float *p)
{
uchar i;
uchar *pc ;

pc = (uchar *)p ; 
for( i=0;i<4;++i)
	{
	out_hex2(*(pc+3-i));	// Serial out one byte as 2 hex characters
	}
}




// Perform a float calculation
FUNCT void calculate(char c)
{
switch ( c )
	{
case '+' :
	hr1 = ha + hb ;
	break ;
case '-' :
	hr1 = ha - hb ;
	break ;
case '*' :
	hr1 = ha * hb ;
	break ;
case '/' :
	hr1 = ha / hb ;
	break ;

case 's' :
	hr1 = sin(ha) ;
	break ;
case 'c' :
	hr1 = cos(ha) ;
	break ;
case 't' :
	hr1 = tan(ha) ;
	break ;
case 'u' :
	hr1 = (float)(ulong)(ha) ;
	break ;
case 'v' :
	hr1 = (float)(slong)(ha) ;
	break ;
default:
	hr1 = ha ;		
	}
}


// The dialog function
FUNCT void fun1(void)
{
char c ;
uint is ;
ulong dtick ;
ulong starttick ;



for(;;)
	{
	out_puts("\nqqtsser $");
	c = in_char() ;
	out_char(c);
	switch ( c )
		{
	case '?' :
		out_puts(
			"\nMenu is: \n"
			"? - Get this help\n"
			"a - Arithmetic Test\n"
			"h - Arithmetic Test (HEX)\n"
			"s - Speed (Time Test)\n"
			"z - Stack-Test\n"
				);
		break ;

	case 'a' :
		c = in_char() ;
		out_char(c);
		in_float( "\nInput ha ?", &ha );
		in_float( "\nInput hb ?", &hb );

		calculate(c);

		out_puts( "\n ha= " );
		out_float(ha);
		out_puts( " hb= " );
		out_float(hb);
		out_puts( " Operation= " );
		out_char(c);
		out_nl();

		out_puts( "\nResult =" ); out_float(hr1);
	
		out_nl();

		break ;

	case 'h' :		// This hexadecimale test is server for the PC qqtstrm.cpp
		c = in_char() ;
		out_char(c);
		in_hex32( "\nInput ha ?", (ulong *)&ha );	// read 8 hex digits, put them as float into ha
		in_hex32( "\nInput hb ?", (ulong *)&hb );	// read 8 hex digits, put them as float into ha
		calculate(c);
// #define OUT_DECIMALE
#ifdef OUT_DECIMALE	// This wastes time at the serial interface - not needed
		out_puts( "\n ha= " );
		out_float(ha);
		out_puts( " hb= " );
		out_float(hb);
		out_nl();
#endif
		out_puts( "\nResult =" ); ohex8(&hr1);

#ifdef OUT_DECIMALE	// This wastes time at the serial interface - not needed
		out_puts( "\n ha= " );
		out_float(hr1);
		out_nl();
#endif
	
		out_nl();

		break ;

	case 's' : // Speed ( time test )
		ha = 7.635431F ;
		hb = 3.145453F ;
		uba = -66 ;
		uwa = 672 ;
		ula = 21341234UL ;
		tsnext("b");
		starttick = gettick();
		dtick = difftick(starttick, gettick());
		out_puts( "\nticks for nothing : " );  out_uint32( dtick );
		starttick = gettick();
		for( is = 0 ; is < AVR_TIMER_DIVIDER ; ++is )
			{
			hr1 = ha + hb ;	// 
			}
		dtick = difftick(starttick, gettick());
		out_puts( "\nticks for ADD : " );  out_uint32( dtick );
		starttick = gettick();
		for( is = 0 ; is < AVR_TIMER_DIVIDER ; ++is )
			{
			hr1 = ha * hb ;	// 
			}
		dtick = difftick(starttick, gettick());
		out_puts( "\nticks for MUL : " );  out_uint32( dtick );
		starttick = gettick();
		for( is = 0 ; is < AVR_TIMER_DIVIDER ; ++is )
			{
			hr1 = ha * hb ;	// 
			}
		dtick = difftick(starttick, gettick());
		out_puts( "\nticks for DIV : " );  out_uint32( dtick );
		starttick = gettick();
		hr7 = (float)ula ;
		dtick = difftick(starttick, gettick());
		out_puts( "\nticks for ulong to float : " );  out_uint32( dtick );
		break ;
	case 'z' :
		for ( is = 0 ; is < NSTK1 ; ++is )
			{
			if ( stk1[is] != (void *)STACK_EMPTY )
				{
				break ;
				}
			}
		out_puts( "\nStack1 unused =" );
		out_uint16( is );
		out_puts( " out of " );
		out_uint16( NSTK1 );
		out_nl();
		break ;

	default:
		out_puts( "\n??? Help=?\n" );
		break ;
		}
	tsnext("fun1");
	}
}






// The main program
FUNCT int main( int argc, const char **argv )
{
uint is ;

tsseropen();

tsopen();		// Clear all tables
tickopen();		// Use the fast timer, for timing calculation

ha = 1.0 ;
hb = 2.0 ;
tsnext("b");
hr1 = ha + hb ;
hr2 = ha * hb ;
hr3 = ha / hb ;
ha = 16.0 ;
hb = 1.0 ;
tsnext("b");
hr1 = ha + hb ;
hr2 = ha * hb ;
hr3 = ha / hb ;
ha = 257.0 ;
hb = 1.0 ;
tsnext("b");
hr1 = ha + hb ;
hr2 = ha * hb ;
hr3 = ha / hb ;




tsnext("b");
// DO early tests for Debugging in Avr Studio
#ifdef TESTCASE_MUL_1
// testcase_mul_1();
#endif
#ifdef TESTCASE_ADD_1
// testcase_add_1();
#endif
#ifdef TESTCASE_DIV_1
// testcase_div_1();
#endif



for ( is = 0 ; is < NSTK1 ; ++is )
	{
	stk1[is] = (void *)STACK_EMPTY ;
	}

out_float(ha);

tseinri( 1 , fun1, "fun1", stk1, sizeof(stk1) );

for(;;)
	{
	tsnext( "main" );	// run one piece of all functions which have been added by tseinri() until they call tsnext()
	}

return 0 ;		// return - will never return !
}


// the three functions call_addsf3, call_mulsf3 and call_divfs3 are only expected to be called
// by the debugger - by qqrsfloa.bas, which is interpreted by simatmel.cpp
// The debugger will preset r21:r20:r19:r18 and r25:r24:r23:r22
extern "C" {
extern float __addsf3();
extern float __mulsf3();
extern float __divsf3();
} ;

void call_addsf3()
{
__addsf3();
__asm( "	mov	r25,r25\n" );	// two statements, so a breakpoint can be set to the position behind
								// the call, for both cases:
								// if the call is a one word instruction (atmgea8535)
								// if the call is a two word instruction (atmega32)
__asm( "	mov	r24,r24\n" );
}

void call_mulsf3()
{
__mulsf3();
__asm( "	mov	r25,r25\n" );
__asm( "	mov	r24,r24\n" );
}

void call_divsf3()
{
__divsf3();
__asm( "	mov	r25,r25\n" );
__asm( "	mov	r24,r24\n" );
}
