// tsaccu - Use the ADC to display Millivolts as they come in
//
// This program uses the functions for serial interface tsser.cpp
// This program is intended to run on a ATMEL microprocessor,
// but has been successfully compiled and run for :
// AVRV : ATMEGA32 at 16 MHz, 19200 Baud Hardware see /home/html/hjh/etronik/s/accutest.htm
// MDOS50 : Just to test the logik

//

#define VERSION "20070916,0.005"


#include <heport.h>
#include <helq.h>
#include <heimath.h>
#include <tshar.h>
#include <tsser.h>
#include <hevt.h>
#include <hegettim.h>
#include <tsinout.h>

#ifdef AVR
#include <avr/io.h>
#endif

#ifdef AVR
#define NSTK1 60
#else
#define NSTK1 2000
#endif

void *stk1[NSTK1] ;		// used 32 out of 60

#ifdef AVR
#define NSTK2 60
#else
#define NSTK2 2000
#endif

void *stk2[NSTK2] ;		// used 26 out of 60

#ifdef AVR
#define NSTKA 60
#else
#define NSTKA 2000
#endif

void *stk3[NSTKA] ;		// used 32 out of 60
void *stk4[NSTKA] ;		// 
void *stk5[NSTKA] ;		// 
void *stk6[NSTKA] ;		// 

#define STACK_EMPTY 0x7EAB	// Pattern

// uword counter ;

#ifdef AVR
		// The stack must not go over a 0x100 page limit
#if ( stk1 & 0xff00 ) != ( stk1+NSTK1 & 0xff00 )
#error "stk1 over page (256) boundary"
#endif

#if ( stk2 & 0xff00 ) != ( stk2+NSTK2 & 0xff00 )
#error "stk2 over page (256) boundary"
#endif

#endif

extern ulong starttick_tsdaytim ;
extern struct sdate_uc date_uc ;
//	  ulong day ;
//	  uchar hour ;
//	  uchar minute ;
//	  uchar second ;

#define NACCU 4
#ifdef _MSC_VER
		sword	raw[NACCU] ;		// Raw ADC Value, channel 0..7
#else
volatile sword	raw[NACCU] ;		// Raw ADC Value, channel 0..7
#endif

#define NMEASTOTAL 16
#define NMEASSTART 4
#define NMEASCOUNT 8
sword rawarray[NACCU][NMEASTOTAL] ;

volatile ulong integ[NACCU] ;		// Fixed point number with dp at position 16
volatile ulong dtime[NACCU] ;		// Discharge time [s]

volatile char status[NACCU] ;

sword endvoltage[NACCU] = { 1400, 1400, 1400, 1400 } ;	// mV
uword endcap[NACCU] = { 1000,1000,1000,1000 } ;		// mAh
volatile bool chargecommand[NACCU] ;
volatile bool testaftercharge[NACCU] ;

volatile uchar adc_cycle ;



// #######################################################################################


FUNCT void out_version(void)
{
out_puts( "\nHJHerbert tsaccu " VERSION "\n" );
}

// The dialog function
FUNCT void fun_dialog(void)
{
char c ;
char a ;
#ifdef _MSC_VER
uchar i ;
#endif
uint is ;
static uchar lhour, lminute, lsecond ;

tsseropen();
out_version();

for(;;)
	{
	c = in_char() ;
	switch ( c )
		{
	case '?' :
		out_version();
		tslcd_help();
		out_puts(
			"Menu is: \n"
			"? - Get help\n"
			"z - Check size of stack\n"
			"t - Display time\n"
			"s - Set time\n"
			"a - Display all 4 ADC actual values\n"
#ifdef _MSC_VER
			"d - Define voltage\n" // The MDOS50 has no ADC
#endif
			"Services are:\n"
			"r - status, adc values, capacity, time\n"
			"cn - Charge accu No 0..3\n"
			"v - Version" );
		break ;
	case 'c' :
		out_puts( "\nCharge Accu [0...3] ?" );
		c = in_char() ;
		if ( c >= '0' && c < '0'+NACCU )
			{
			out_char( c );
			in_int16( "\nWhich maximum charge voltage [mV] ?" , &endvoltage[c-'0'] );
			in_uint16( "\nWhich maximum charge capacity [mAh] ?" , &endcap[c-'0'] );
			out_puts( "\nCheck and discharge after charge ? [y/n] ?" );
			a = in_char() ;
			if ( a == 'y' ) testaftercharge[c-'0'] = true ;
			else testaftercharge[c-'0'] = false ;
			chargecommand[c-'0'] = true ;
			out_nl();
			}
		else
			{
			out_puts( "? Bad Cell Number, allowed 0..3\n" );
			}
		break ;
	case 'v' :
		out_version();
		break ;
	case 'z' :
		for ( is = 0 ; is < NSTK1 ; ++is )
			{
			if ( stk1[is] != (void *)STACK_EMPTY )
				{
				break ;
				}
			}
		out_puts( "\nStack1 unused =" );
		out_uint16( is );
		out_puts( " out of " );
		out_uint16( NSTK1 );
		out_nl();

		for ( is = 0 ; is < NSTK2 ; ++is )
			{
			if ( stk2[is] != (void *)STACK_EMPTY )
				{
				break ;
				}
			}
		out_puts( "\nStack2 unused =" );
		out_uint16( is );
		out_puts( " out of " );
		out_uint16( NSTK2 );
		out_nl();

		for ( is = 0 ; is < NSTKA ; ++is )
			{
			if ( stk3[is] != (void *)STACK_EMPTY )
				{
				break ;
				}
			}
		out_puts( "\nStack3 unused =" );
		out_uint16( is );
		out_puts( " out of " );
		out_uint16( NSTKA );
		out_nl();

		break ;
	case 't' :
		out_puts( "\nTime=" );
		out_uint32( date_uc.day );
		out_char( '-' );
		out_uint8( date_uc.hour );
		out_char( ':' );
		out_uint8( date_uc.minute );
		out_char( ':' );
		out_uint8( date_uc.second );
		break ;

	case 's' :
			if ( in_uint8( " Hour ?", &lhour ) == 0 &&
				 in_uint8( " Minute ?", &lminute ) == 0 &&
				 in_uint8( " Second ?", &lsecond ) == 0 )
				{	// Time entered without error
				starttick_tsdaytim = gettick() ;
				date_uc.minute = lminute ;
				date_uc.second = lsecond ;
				date_uc.hour   = lhour ;
				date_uc.day	   = 0 ;
				}
		break ;

#ifdef _MSC_VER
		// This is for the logic test only
	case 'd' :		// Simulate (Input) the ADC value
		if ( in_uint8( " channel [0..3]", &i ) == 0 )
			{
			if ( i >= 0 && i <= 3 )
				{
				sword mvval ;
				if ( in_int16( " Voltage [mV]", &mvval ) == 0 )
					{
					raw[i] = mvolt_to_adc( i, mvval ) ;
					for ( int j = 0 ; j < NMEASTOTAL-1 ; ++j )
						rawarray[i][j] = raw[i] ;
					}
				}
			}
		break ;
#endif

	case 'a' :
		out_puts( "\nStatus and values :\n" );
		for ( is = 0 ; is < NACCU ; ++is )
			{
			out_uint8( is );
			out_char( status[is] );
			out_char( '=' );

			out_int16( adc_to_mvolt( is, raw[is] ) ); // 2.56 Volt = vref = 0xffc0
			out_puts( "mV  " );

			out_int16( (uword)( integ[is] >> 16) ); 		
			out_puts( "mAh  " );

			out_uint32( (uword)(dtime[is]) ); 		
			out_puts( "s  " );

			out_nl();
			}
		break ;

	case 'r' :
		for ( is = 0 ; ; ++is )
			{
			out_char( status[is] );
			out_char( ',' );

			out_int16( adc_to_mvolt( is, raw[is] ) ); 		// Voltage in Millivolts
			out_char( ',' );

			out_int16( (uword)( integ[is] >> 16) ); 	// capacity in mAh
			out_char( ',' );

			out_uint32( (uword)(dtime[is]) ); 			// Discharge time in seconds
			if ( is == NACCU-1 ) break ;
			out_char( ',' );

			}
		out_nl();
		break ;

case 0x0d:
case 0x0a:
case 0x09:
case 0x20:
		break ;
	default:
		out_puts( "\n* Illegal Command, Help=?\n" );
		break ;
		}
	out_puts("\n");
	tsnext("fun_dialog");
	}
}



// This task displays 
//   - the actual Voltages in the upper LCD row 
//   - the capacity in the lower LCD row

FUNCT void fun_lcd( void )
{
int m ;
ulong startcycle ;
uchar phase = 0 ;


tslcd_init();
vout_char[tshar_task_no] = tslcd_ocha ;	// To LCD instead to serial port
// table_in_char[tshar_task_no] = tsser_in_char ;
// table_in_check[tshar_task_no] = tsser_in_check ;
out_version();
delaytick( 4000 );		// Let the Version displayed for some milliseconds
tslcd_cls();

startcycle = gettick();
for(;;)
	{
	tsnext( "nomean" );
	if ( difftick( startcycle, gettick() ) > msectotick( 1500 ) )
		{
		startcycle += msectotick( 1500 ) ;
		// 1500 msec are gone


		for ( m = 0 ; m < NACCU ; ++m )
			{
			tslcd_pos( m*6, 0 );
			out_puts( "        " );
			tslcd_pos( m*6, 0 );
			out_char( status[m] );
			out_int16( adc_to_mvolt( m, raw[m] ) ); 	// 2.56 Volt = vref = 0xffc0
			}
		// Display the capacity for 1.5 seconds
		for ( m = 0 ; m < NACCU ; ++m )
			{
			tslcd_pos( m*6, 1 );
			out_puts( "        " );
			tslcd_pos( m*6, 1 );
			if ( phase == 0 )
				{
				out_int16( (sword)( integ[m] >> 16) ); 	// Milliamperehours
				out_char( 'c' );
				}
			else
				{
				out_uint32( dtime[m] ); 	// Milliamperehours
				out_char( 's' );
				}
			}
		
		if ( phase == 0 )
			{
			phase = 1 ;
			}
		else
			{
			phase = 0 ;
			}
			
		}
	tsnext("lcd");
	}
}



static int m0 = 0 ;



#ifdef AVR
#define CHARGE      PORTC &= ~(1<<(m*2)) ;		// do charge
#define NOCHARGE    PORTC |=  (1<<(m*2)) ;		// do not charge

#define NODISCHARGE   PORTC &= ~(2<<(m*2)) ; 		// Do Discharge
#define DISCHARGE PORTC |=  (2<<(m*2)) ; 		// Do not Discharge
	
#endif


FUNCT void fun_accutest(void)
{
int m ;
ulong starttick ;
ulong errortick ;
schar nchecktime ;
uchar ladc_cycle ;
#define NCHECKTIME 50	// Every 50 seconds check the voltage of a idling running accu

m = m0 ;			// m = 0 ... 3 for tshar_task_no = 
++m0 ;
#ifdef AVR
DDRC |= (3<<(m*2)) ;			// All Output
#endif


// D0 = charge when Output = low
// D1 = discharege when output = low
for(;;)
	{
	status[m] = 'i' ;
// The idle status - No accu is set in
#ifdef AVR
	NOCHARGE ;
	NODISCHARGE ;
#endif
	starttick = gettick();
	for(;;)
		{
		if ( adc_to_mvolt( m, raw[m] ) < 1160 )
			{
			starttick = gettick();
			}
		else
			{
			if ( difftick( starttick, gettick() ) > msectotick( 1500 ) ) break ;
			}
		tsnext("idle");			// wait for accu to come
		if ( chargecommand[m] ) goto MCharge ;		// Status i to Status c
		}		// End of the inner for loop which is left if more than 5 seconds
	// An accu is present
	starttick = gettick();
// The discharge begin status
#ifdef AVR
	NOCHARGE ;
	DISCHARGE ;
	
#endif
	status[m] = 't' ;
	delaytick( msectotick(2000) );
	if ( adc_to_mvolt( m, raw[m]) > 1080 ) // Even after 2 seconds of 200 mA the accu has some voltage
		{
		// 
		integ[m] = 0 ;
		dtime[m] = 0 ;
		status[m] = 'd' ;					// Discharging
		starttick = gettick();
		nchecktime = NCHECKTIME ;
		for(;;)
			{
			if ( chargecommand[m] ) goto MCharge ;		// Status d to Status c
			if ( difftick( starttick, gettick() ) > msectotick( 1000 ) )
				{		// Every 1000 msec
				starttick = addtick( starttick, msectotick( 1000 ) );
				if ( --nchecktime == 0 )
					{
#ifdef AVR
					NODISCHARGE ;
#endif
					status[m] = 'm' ;			// Measureing without load
					errortick = gettick();		// Start of pause
					ladc_cycle = adc_cycle + 2 ;	
					while ( ladc_cycle != adc_cycle )
						{						// Wait until the mean value is made only of values from I=0
						tsnext("Wvi0" );
						}
					if ( adc_to_mvolt( m, raw[m] ) < 1000 )
						{
						delaytick( msectotick(4000) );		// Wait four seconds, until nobody plays with the contacts
						if ( adc_to_mvolt( m, raw[m] ) < 1000 )
							{				// Even in the next second the value is too low
							break ;			// So stop dicharging
							}
						}
#ifdef AVR
					DISCHARGE ; 		// Do Discharge again
#endif
					status[m] = 'd' ;					// Discharging
					starttick = addtick( starttick, difftick( errortick, gettick() ) );	//
					ladc_cycle = adc_cycle + 2 ;	
					while ( ladc_cycle != adc_cycle )
						{						// Wait until the mean value is made only of values from I = discharge current
						tsnext("Wvid" );
						}
					nchecktime = NCHECKTIME ;
					}
				dtime[m] += 1 ;		// one more seconds

/*				integ[m] += (adc_to_mvolt( m, raw[m] ) - 100)  * 20.0/3600.0 / 0.006 ;
						// 20.0 : Seconds step to step of integrator
						// 3600 : seconds per hour
						// 6 = 6 Ohm discharge resistor
						// because we multiply by millivolts so we get milliamperehours
						// * 65536.0 = fixed point 1.0
			The multiplication / division by 2000 makes the influence of rounding errors smaller.
*/
// 				integ[m] += (adc_to_mvolt( m, raw[m] ) - 100)  * 5.0/3600.0 / 6.0 * 65536 ;

#ifdef _MSC_VER
				uword addc = (uword)(1.0/3600.0 / 6.0 * 65536.0 *4) ;	// 242.726
#endif

				integ[m] += div_ul_uw (
								mul_uw_uw( (adc_to_mvolt( m, raw[m] ) - 13), (uword)(1.0/3600.0 / 7.13 * 65536.0 *2000.0) ),
									2000L );



				}
			tsnext( "discharge" );
			} 
		}
	// The accu is discharged
#ifdef AVR
	NOCHARGE ;
	NODISCHARGE ;
#endif
	status[m] = 'f' ;			// Finished
	starttick = gettick();
	for(;;)
		{
		if ( chargecommand[m] ) goto MCharge ;		// Status f to Status c
		
		if ( adc_to_mvolt( m, raw[m] ) > 500 )
			{
			starttick = gettick();
			}
		else
			{
			if ( difftick( starttick, gettick() ) > msectotick( 5000 ) ) break ;
			}
		tsnext("idle");			// wait for accu to come			
		}		// End of the inner for loop which is left if more than 5 seconds
	continue ;



// Charge C


MCharge:
	integ[m] = 0 ;
	dtime[m] = 0 ;
	chargecommand[m] = false ;
	status[m] = 'c' ;			// Charge
#ifdef AVR
	CHARGE ;
	NODISCHARGE ;
#endif
	starttick = gettick();
	for(;;)
		{


		if ( difftick( starttick, gettick() ) > msectotick( 1000 ) )
			{		// Every 1000 msec
			starttick = addtick( starttick, msectotick( 1000 ) );
			dtime[m] += 1 ;		// one more seconds
			integ[m] += div_ul_uw (
							mul_uw_uw( 5000 - (adc_to_mvolt( m, raw[m] ) ), (uword)(1.0/3600.0 / 22.0 * 65536.0 *2000.0) ),
								2000L );
			if ( adc_to_mvolt( m, raw[m] ) >= endvoltage[m] )
				{
				break ;
				}
			if ( ( integ[m]>>16 ) > endcap[m] ) break ;
			}
		tsnext("charge");			// wait for accu to come			
		}		// 

	chargecommand[m] = false ;		// HJH20070916

#ifdef AVR
	NOCHARGE ;
	NODISCHARGE ;
#endif
		
	if ( testaftercharge[m] )
		{
		testaftercharge[m] = false ;
		continue ;
		}
	// Else wait until the accu is removed
	status[m] = 'w' ;			// Wait after Charge
	while( adc_to_mvolt( m, raw[m] ) > 1100 )
		{
		tsnext( "wait" );
		}	
	}
}




// The main program
FUNCT int main( int argc, const char **argv )
{
uint is ;
uword j,k ;
int m ;
ulong starttick ;
void *se ;
sword ref ;

se = (void *)STACK_EMPTY ;

for ( is = 0 ; is < NSTK1 ; ++is )
	{
	stk1[is] = se ;
	}
for ( is = 0 ; is < NSTK2 ; ++is )
	{
	stk2[is] = se ;
	}
for ( is = 0 ; is < NSTKA ; ++is )
	{
	stk3[is] = se ;
	stk4[is] = se ;
	stk5[is] = se ;
	stk6[is] = se ;
	}

tsopen();		// Clear all tables
tickopen();

tseinri( 1 , fun_dialog, "fun_dialog", stk1, sizeof(stk1) );
tseinri( 2 , fun_lcd, "fun_lcd", stk2, sizeof(stk2) );
tseinri( 3 , fun_accutest, "fun_accutest", stk3, sizeof(stk3) );
tseinri( 4 , fun_accutest, "fun_accutest", stk4, sizeof(stk4) );
tseinri( 5 , fun_accutest, "fun_accutest", stk5, sizeof(stk5) );
tseinri( 6 , fun_accutest, "fun_accutest", stk6, sizeof(stk6) );

tsdaytim_open();

for(;;)
	{
	tsnext( "main" );	// run one time slice of fun_dialog and fun_lcd and fun_accutest
	tsdaytim();		// keep running g_msec and the Daytime
	starttick = gettick();

	for( j = 0 ; j < NMEASTOTAL ; ++j )
		{
		ref = adc(1) ;
		for( m = 0 ; m < NACCU ; ++m )
			{
			rawarray[m][j] = (adc(4+m)-ref)/NMEASCOUNT ;	// 4+m = measure agauinst Gnd
															// 20+m = measure against adc1
			}
		while ( difftick( starttick, gettick() ) < msectotick( 20 ) )
			{										// Wait 20 msec
			tsnext("main");
			}
		starttick += msectotick( 20 );
		}

// Sort the array for all NACCU channels
	for( m = 0 ; m < NACCU ; ++m )
		{
		for ( j = 0 ; j < NMEASTOTAL-1 ; ++j )
			{
			for ( k = j ; ++k < NMEASTOTAL ; )
				{
				if ( rawarray[m][j] > rawarray[m][k] )
					{
					sword z ;
					z               = rawarray[m][j] ;
					rawarray[m][j] = rawarray[m][k] ;
					rawarray[m][k] = z ;
					}
				}
			}
		sword sum = 0 ;
		for ( j = NMEASSTART ; j < NMEASSTART+NMEASCOUNT ; ++j )
			{
			sum += rawarray[m][j] ;
			}
		raw[m] = sum ;
		}
	++adc_cycle ;

	}
return 0 ;		// return - will never return !
}

